@@ -1,5 +1,5 @@
 /* dotfile.c --- management of the ~/.xscreensaver file.
- * xscreensaver, Copyright (c) 1998-2008 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright (c) 1998-2020 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -64,6 +64,7 @@
 #endif
 
 
+#include "version.h"
 #include "prefs.h"
 #include "resources.h"
 
@@ -135,33 +136,34 @@ init_file_name (void)
   if (!file)
     {
       uid_t uid = getuid ();
-      struct passwd *p = getpwuid (uid);
+      const char *home = getenv("HOME");
 
-      if (i_am_a_nobody (uid))
-        /* If we're running as nobody, then use root's .xscreensaver file
-           (since ~root/.xscreensaver and ~nobody/.xscreensaver are likely
-           to be different -- if we didn't do this, then xscreensaver-demo
-           would appear to have no effect when the luser is running as root.)
-         */
-        uid = 0;
-
-      p = getpwuid (uid);
-
-      if (!p || !p->pw_name || !*p->pw_name)
+      if (i_am_a_nobody (uid) || !home || !*home)
 	{
-	  fprintf (stderr, "%s: couldn't get user info of uid %d\n",
-		   blurb(), getuid ());
-	  file = "";
+	  /* If we're running as nobody, then use root's .xscreensaver file
+	     (since ~root/.xscreensaver and ~nobody/.xscreensaver are likely
+	     to be different -- if we didn't do this, then xscreensaver-demo
+	     would appear to have no effect when the luser is running as root.)
+	   */
+          struct passwd *p = getpwuid (uid);
+	  uid = 0;
+	  if (!p || !p->pw_name || !*p->pw_name)
+	    {
+	      fprintf (stderr, "%s: couldn't get user info of uid %d\n",
+		       blurb(), getuid ());
+	    }
+	  else if (!p->pw_dir || !*p->pw_dir)
+	    {
+	      fprintf (stderr, "%s: couldn't get home directory of \"%s\"\n",
+		       blurb(), (p->pw_name ? p->pw_name : "???"));
+	    }
+	  else
+	    {
+	      home = p->pw_dir;
+	    }
 	}
-      else if (!p->pw_dir || !*p->pw_dir)
+      if (home && *home)
 	{
-	  fprintf (stderr, "%s: couldn't get home directory of \"%s\"\n",
-		   blurb(), (p->pw_name ? p->pw_name : "???"));
-	  file = "";
-	}
-      else
-	{
-	  const char *home = p->pw_dir;
 	  const char *name = ".xscreensaver";
 	  file = (char *) malloc(strlen(home) + strlen(name) + 2);
 	  strcpy(file, home);
@@ -169,6 +171,10 @@ init_file_name (void)
 	    strcat(file, "/");
 	  strcat(file, name);
 	}
+      else
+	{
+	  file = "";
+	}
     }
 
   if (file && *file)
@@ -276,6 +282,7 @@ static const char * const prefs[] = {
   "ignoreUninstalledPrograms",
   "font",
   "dpmsEnabled",
+  "dpmsQuickOff",
   "dpmsStandby",
   "dpmsSuspend",
   "dpmsOff",
@@ -297,15 +304,17 @@ static const char * const prefs[] = {
   "pointerHysteresis",
   "windowCreationTimeout",
   "initialDelay",
-  "sgiSaverExtension",
+  "sgiSaverExtension",		/* not saved -- obsolete */
   "mitSaverExtension",		/* not saved -- obsolete */
-  "xidleExtension",
+  "xidleExtension",		/* not saved -- obsolete */
   "GetViewPortIsFullOfLies",
   "procInterrupts",
+  "xinputExtensionDev",
   "overlayStderr",
   "overlayTextBackground",	/* not saved -- X resources only */
   "overlayTextForeground",	/* not saved -- X resources only */
   "bourneShell",		/* not saved -- X resources only */
+  "authWarningSlack",
   0
 };
 
@@ -751,7 +760,7 @@ write_init_file (Display *dpy,
     fprintf (out,
 	     "# %s Preferences File\n"
 	     "# Written by %s %s for %s on %s.\n"
-	     "# http://www.jwz.org/xscreensaver/\n"
+	     "# https://www.jwz.org/xscreensaver/\n"
 	     "\n",
 	     progclass, progname, version_string, whoami, timestr);
   }
@@ -816,6 +825,7 @@ write_init_file (Display *dpy,
       CHECK("font")		type = pref_str,  s =    stderr_font;
 
       CHECK("dpmsEnabled")	type = pref_bool, b = p->dpms_enabled_p;
+      CHECK("dpmsQuickOff")	type = pref_bool, b = p->dpms_quickoff_p;
       CHECK("dpmsStandby")	type = pref_time, t = p->dpms_standby;
       CHECK("dpmsSuspend")	type = pref_time, t = p->dpms_suspend;
       CHECK("dpmsOff")		type = pref_time, t = p->dpms_off;
@@ -850,16 +860,18 @@ write_init_file (Display *dpy,
       CHECK("pointerHysteresis")type = pref_int,  i = p->pointer_hysteresis;
       CHECK("windowCreationTimeout")type=pref_time,t= p->notice_events_timeout;
       CHECK("initialDelay")	type = pref_time, t = p->initial_delay;
-      CHECK("sgiSaverExtension")type = pref_bool, b=p->use_sgi_saver_extension;
+      CHECK("sgiSaverExtension") continue;  /* don't save */
       CHECK("mitSaverExtension") continue;  /* don't save */
-      CHECK("xidleExtension")	type = pref_bool, b = p->use_xidle_extension;
+      CHECK("xidleExtension")	 continue;  /* don't save */
       CHECK("procInterrupts")	type = pref_bool, b = p->use_proc_interrupts;
+      CHECK("xinputExtensionDev") type = pref_bool, b = p->use_xinput_extension;
       CHECK("GetViewPortIsFullOfLies")  type = pref_bool,
 					b = p->getviewport_full_of_lies_p;
       CHECK("overlayStderr")	type = pref_bool, b = overlay_stderr_p;
       CHECK("overlayTextBackground") continue;  /* don't save */
       CHECK("overlayTextForeground") continue;  /* don't save */
-      CHECK("bourneShell")	continue;
+      CHECK("bourneShell")	     continue;  /* don't save */
+      CHECK("authWarningSlack") type = pref_int, i = p->auth_warning_slack;
       else			abort();
 # undef CHECK
 
@@ -955,7 +967,9 @@ write_init_file (Display *dpy,
 
 	  /* Since the .xscreensaver file is used for IPC, let's try and make
 	     sure that the bits actually land on the disk right away. */
-	  sync ();
+          /* Update 2020: Apparently here in the future, this sometimes takes
+             3+ seconds, so let's not. */
+	  /* sync(); */
 
           status = 0;    /* wrote and renamed successfully! */
 	}
@@ -1071,6 +1085,7 @@ load_init_file (Display *dpy, saver_preferences *p)
 						       "Time");
 
   p->dpms_enabled_p  = get_boolean_resource (dpy, "dpmsEnabled", "Boolean");
+  p->dpms_quickoff_p = get_boolean_resource (dpy, "dpmsQuickOff", "Boolean");
   p->dpms_standby    = 1000 * get_minutes_resource (dpy, "dpmsStandby", "Time");
   p->dpms_suspend    = 1000 * get_minutes_resource (dpy, "dpmsSuspend", "Time");
   p->dpms_off        = 1000 * get_minutes_resource (dpy, "dpmsOff",     "Time");
@@ -1096,6 +1111,8 @@ load_init_file (Display *dpy, saver_preferences *p)
   p->new_login_command = get_string_resource(dpy,
                                              "newLoginCommand",
                                              "NewLoginCommand");
+  p->auth_warning_slack = get_integer_resource(dpy, "authWarningSlack",
+                                               "Integer");
 
   /* If "*splash" is unset, default to true. */
   {
@@ -1116,14 +1133,21 @@ load_init_file (Display *dpy, saver_preferences *p)
   }
 
   p->use_xidle_extension = get_boolean_resource (dpy, "xidleExtension","Boolean");
-#if 0 /* ignore this, it is evil. */
+#if 0 /* obsolete. */
+  p->use_sgi_saver_extension = get_boolean_resource (dpy,
+                                                     "sgiSaverExtension",
+						     "Boolean");
+#endif
+#ifdef HAVE_XINPUT
+  p->use_xinput_extension = get_boolean_resource (dpy, "xinputExtensionDev",
+                                                  "Boolean");
+#endif
+#if 0 /* broken and evil. */
   p->use_mit_saver_extension = get_boolean_resource (dpy, 
                                                      "mitSaverExtension",
 						     "Boolean");
 #endif
-  p->use_sgi_saver_extension = get_boolean_resource (dpy,
-                                                     "sgiSaverExtension",
-						     "Boolean");
+
   p->use_proc_interrupts = get_boolean_resource (dpy,
                                                  "procInterrupts", "Boolean");
 
@@ -1622,7 +1646,9 @@ stop_the_insanity (saver_preferences *p)
 
   if (p->dpms_standby == 0 &&	   /* if *all* are 0, then DPMS is disabled */
       p->dpms_suspend == 0 &&
-      p->dpms_off     == 0)
+      p->dpms_off     == 0 &&
+      !(p->dpms_quickoff_p)           /* ... but we want to do DPMS quick off */
+      )
     p->dpms_enabled_p = False;
 
 
@@ -1634,5 +1660,118 @@ stop_the_insanity (saver_preferences *p)
   if (p->watchdog_timeout > 57000) p->watchdog_timeout = 57000;   /* 57 secs */
 
   if (p->pointer_hysteresis < 0)   p->pointer_hysteresis = 0;
-  if (p->pointer_hysteresis > 100) p->pointer_hysteresis = 100;
+/* if (p->pointer_hysteresis > 100) p->pointer_hysteresis = 100; */
+
+  if (p->auth_warning_slack < 0)   p->auth_warning_slack = 0;
+  if (p->auth_warning_slack > 300) p->auth_warning_slack = 300;
+}
+
+
+Bool
+senesculent_p (void)
+{
+  /* If you are in here because you're planning on disabling this warning
+     before redistributing my software, please don't.
+
+     I sincerely request that you do one of the following:
+
+         1: leave this code intact and this warning in place, -OR-
+
+         2: Remove xscreensaver from your distribution.
+
+     I would seriously prefer that you not distribute my software at all
+     than that you distribute one version and then never update it for
+     years.
+
+     I am *constantly* getting email from users reporting bugs that have
+     been fixed for literally years who have no idea that the software
+     they are running is years out of date.  Yes, it would be great if we
+     lived in the ideal world where people checked that they were running
+     the latest release before they report a bug, but we don't.  To most
+     people, "running the latest release" is synonymous with "running the
+     latest release that my distro packages for me."
+
+     When they even bother to tell me what version they're running, I
+     say, "That version is three years old!", and they say "But this is
+     the latest version my distro ships".  Then I say, "your distro
+     sucks", and they say "but I don't know how to compile from source,
+     herp derp I eat paste", and *everybody* goes away unhappy.
+
+     It wastes an enormous amount of my time, but worse than that, it
+     does a grave disservice to the users, who are stuck experiencing
+     bugs that are already fixed!  These users think they are running the
+     latest release, and they are not.  They would like to be running the
+     actual latest release, but they don't know how, because their distro
+     makes that very difficult for them.  It's terrible for everyone, and
+     kind of makes me regret ever having released this software in the
+     first place.
+
+     So seriously. I ask that if you're planning on disabling this
+     obsolescence warning, that you instead just remove xscreensaver from
+     your distro entirely.  Everybody will be happier that way.  Check
+     out gnome-screensaver instead, I understand it's really nice.
+
+     Of course, my license allows you to ignore me and do whatever the
+     fuck you want, but as the author, I hope you will have the common
+     courtesy of complying with my request.
+
+     Thank you!
+
+     jwz, 2014, 2016, 2018.
+
+     PS: In particular, since Debian refuses to upgrade software on any
+     kind of rational timeline, I have asked that they stop shipping
+     xscreensaver at all.  They have refused.  Instead of upgrading the
+     software, they simply patched out this warning.
+
+     If you want to witness the sad state of the open source peanut
+     gallery, look no farther than the comments on my blog:
+     http://jwz.org/b/yiYo
+
+     Many of these people fall back on their go-to argument of, "If it is
+     legal, it must be right."  If you believe in that rhetorical device
+     then you are a terrible person, and possibly a sociopath.
+
+     There are also the armchair lawyers who say "Well, instead of
+     *asking* people to do the right thing out of common courtesy, you
+     should just change your license to prohibit them from acting
+     amorally."  Again, this is the answer of a sociopath, but that aside,
+     if you devote even a second's thought to this you will realize that
+     the end result of this would be for distros like Debian to just keep
+     shipping the last version with the old license and then never
+     upgrading it again -- which would be the worst possible outcome for
+     everyone involved, most especially the users.
+  */
+
+  time_t now = time ((time_t *) 0);				/*   d   */
+  struct tm *tm = localtime (&now);				/*   o   */
+  const char *s = screensaver_id;				/*   n   */
+  char mon[4], year[5];						/*   '   */
+  int m, y, mrnths;						/*   t   */
+  s = strchr (s, ' '); if (!s) abort(); s++;			/*       */
+  s = strchr (s, '('); if (!s) abort(); s++;			/*   d   */
+  s = strchr (s, '-'); if (!s) abort(); s++;			/*   o   */
+  strncpy (mon, s, 3);						/*   o   */
+  mon[3] = 0;							/*       */
+  s = strchr (s, '-'); if (!s) abort(); s++;			/*   e   */
+  strncpy (year, s, 4);						/*   e   */
+  year[4] = 0;							/*   t   */
+  y = atoi (year);						/*   ,   */
+  if      (!strcmp(mon, "Jan")) m = 0;				/*       */
+  else if (!strcmp(mon, "Feb")) m = 1;				/*   s   */
+  else if (!strcmp(mon, "Mar")) m = 2;				/*   t   */
+  else if (!strcmp(mon, "Apr")) m = 3;				/*   o   */
+  else if (!strcmp(mon, "May")) m = 4;				/*   p   */
+  else if (!strcmp(mon, "Jun")) m = 5;				/*   ,   */
+  else if (!strcmp(mon, "Jul")) m = 6;				/*       */
+  else if (!strcmp(mon, "Aug")) m = 7;				/*   s   */
+  else if (!strcmp(mon, "Sep")) m = 8;				/*   t   */
+  else if (!strcmp(mon, "Oct")) m = 9;				/*   a   */
+  else if (!strcmp(mon, "Nov")) m = 10;				/*   a   */
+  else if (!strcmp(mon, "Dec")) m = 11;				/*   a   */
+  else abort();							/*   h   */
+  mrnths = ((((tm->tm_year + 1900) * 12) + tm->tm_mon) -	/*   h   */
+            (y * 12 + m));					/*   h   */
+							  	/*   p   */
+  return (mrnths >= 17);					/*   .   */
 }
