@@ -1,5 +1,5 @@
 /* screens.c --- dealing with RANDR, Xinerama, and VidMode Viewports.
- * xscreensaver, Copyright (c) 1991-2008 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright (c) 1991-2020 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -284,6 +284,23 @@ vidmode_scan_monitors (Display *dpy, char **errP)
           m->height = ml.vdisplay;
         }
 
+      /* On a system that has VidMode but does not have RANDR, and that has
+         "Option Rotate" set, WidthOfScreen/HeightOfScreen are the rotated
+         size, but XF86VidModeModeLine contains the unrotated size.
+         Maybe there's something in 'flags' that indicates this?
+         Or, we can just notice that the aspect ratios are inverted:
+       */
+      if (m->width > 0 &&
+          m->height > 0 &&
+          ((m->width > m->height) != 
+           (WidthOfScreen(screen) > HeightOfScreen(screen))))
+        {
+          int swap = m->width;
+          m->width = m->height;
+          m->height = swap;
+        }
+
+
       /* Apparently, though the server stores the X position in increments of
          1 pixel, it will only make changes to the *display* in some other
          increment.  With XF86_SVGA on a Thinkpad, the display only updates
@@ -294,7 +311,7 @@ vidmode_scan_monitors (Display *dpy, char **errP)
          I consider it a bug that XF86VidModeGetViewPort() is telling me the
          server's *target* scroll position rather than the server's *actual*
          scroll position.  David Dawes agrees, and says they may fix this in
-         XFree86 4.0, but it's notrivial.
+         XFree86 4.0, but it's nontrivial.
 
          He also confirms that this behavior is server-dependent, so the
          actual scroll position cannot be reliably determined by the client.
@@ -436,7 +453,7 @@ randr_scan_monitors (Display *dpy, char **errP)
               XRROutputInfo *rroi = XRRGetOutputInfo (dpy, res, 
                                                       res->outputs[k]);
               RRCrtc crtc = (rroi->crtc  ? rroi->crtc :
-                             rroi->crtcs ? rroi->crtcs[0] : 0);
+                             rroi->ncrtc ? rroi->crtcs[0] : 0);
               XRRCrtcInfo *crtci = (crtc ? XRRGetCrtcInfo(dpy, res, crtc) : 0);
 
               monitors[j] = m;
@@ -559,7 +576,7 @@ randr_versus_xinerama_fight (Display *dpy, monitor **randr_monitors,
     {
       *errP = append (*errP,
                       "WARNING: RANDR reports 1 screen but Xinerama\n"
-                      "\t\treports multiple.  Believing Xinerama.");
+                      "         reports multiple.  Believing Xinerama.");
       free_monitors (randr_monitors);
       return xinerama_monitors;
     }
@@ -567,7 +584,7 @@ randr_versus_xinerama_fight (Display *dpy, monitor **randr_monitors,
     {
       *errP = append (*errP,
                       "WARNING: RANDR and Xinerama report different\n"
-                      "\t\tscreen layouts!  Believing RANDR.");
+                      "         screen layouts!  Believing RANDR.");
       free_monitors (xinerama_monitors);
       return randr_monitors;
     }
@@ -1049,8 +1066,8 @@ update_screen_layout (saver_info *si)
       if (ssi->screen != old_screen)
         initialize_screen_root_widget (ssi);
 
-      ssi->poll_mouse_last_root_x = -1;
-      ssi->poll_mouse_last_root_y = -1;
+      ssi->last_poll_mouse.root_x = -1;
+      ssi->last_poll_mouse.root_y = -1;
 
       ssi->x      = m->x;
       ssi->y      = m->y;
@@ -1072,6 +1089,22 @@ update_screen_layout (saver_info *si)
       j++;
     }
 
+  for (; j < count; j++)
+    {
+      saver_screen_info *ssi = &si->screens[j];
+      if (!ssi->screensaver_window)
+        continue;
+      fprintf (stderr, "%s: %d: screen now unused, disabling.\n",
+               blurb(), j);
+      /* Undo store_saver_id() so that xscreensaver-command doesn't attempt
+         to communicate with us through this window. It might make more
+         sense to destroy the window, but I'm not 100% sure that there are
+         no outstanding grabs on it that have yet been transferred.
+       */
+      XDeleteProperty (si->dpy, ssi->screensaver_window,
+                       XA_SCREENSAVER_VERSION);
+    }
+
   si->default_screen = &si->screens[0];
   return True;
 }
