@@ -1,5 +1,5 @@
 /* subprocs.c --- choosing, spawning, and killing screenhacks.
- * xscreensaver, Copyright (c) 1991-2016 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright (c) 1991-2020 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -44,6 +44,7 @@
 #endif /* VMS */
 
 #include <signal.h>		/* for the signal names */
+#include <time.h>
 
 #if !defined(SIGCHLD) && defined(SIGCLD)
 # define SIGCHLD SIGCLD
@@ -61,7 +62,7 @@ extern int kill (pid_t, int);		/* signal() is in sys/signal.h... */
 #undef XtPointer
 #define XtAppContext void*
 #define XrmDatabase  void*
-#define XtIntervalId void*
+#define XtIntervalId unsigned long
 #define XtPointer    void*
 #define Widget       void*
 
@@ -259,13 +260,12 @@ make_job (pid_t pid, int screen, const char *cmd)
   const char *in = cmd;
   char *out = name;
   int got_eq = 0;
-  int first = 1;
 
   clean_job_list();
 
  AGAIN:
-  while (isspace(*in)) in++;		/* skip whitespace */
-  while (!isspace(*in) && *in != ':') {
+  while (*in && isspace(*in)) in++;		/* skip whitespace */
+  while (*in && !isspace(*in) && *in != ':') {
     if (*in == '=') got_eq = 1;
     *out++ = *in++;			/* snarf first token */
   }
@@ -274,11 +274,10 @@ make_job (pid_t pid, int screen, const char *cmd)
     {					/* then get the next token instead. */
       got_eq = 0;
       out = name;
-      first = 0;
       goto AGAIN;
     }
 
-  while (isspace(*in)) in++;		/* skip whitespace */
+  while (*in && isspace(*in)) in++;		/* skip whitespace */
   *out = 0;
 
   job->name = strdup(name);
@@ -370,7 +369,8 @@ find_job (pid_t pid)
 
 static void await_dying_children (saver_info *si);
 #ifndef VMS
-static void describe_dead_child (saver_info *, pid_t, int wait_status);
+static void describe_dead_child (saver_info *, pid_t, int wait_status,
+                                 struct rusage);
 #endif
 
 
@@ -440,7 +440,7 @@ unblock_sigchld (void)
   block_sigchld_handler--;
 }
 
-static int
+int
 kill_job (saver_info *si, pid_t pid, int signal)
 {
   saver_preferences *p = &si->prefs;
@@ -562,9 +562,10 @@ await_dying_children (saver_info *si)
     {
       int wait_status = 0;
       pid_t kid;
+      struct rusage rus;
 
       errno = 0;
-      kid = waitpid (-1, &wait_status, WNOHANG|WUNTRACED);
+      kid = wait4 (-1, &wait_status, WNOHANG|WUNTRACED, &rus);
 
       if (si->prefs.debug_p)
 	{
@@ -601,13 +602,14 @@ await_dying_children (saver_info *si)
 	  (kid < 0 && errno != EINTR))
 	break;
 
-      describe_dead_child (si, kid, wait_status);
+      describe_dead_child (si, kid, wait_status, rus);
     }
 }
 
 
 static void
-describe_dead_child (saver_info *si, pid_t kid, int wait_status)
+describe_dead_child (saver_info *si, pid_t kid, int wait_status,
+                     struct rusage rus)
 {
   int i;
   saver_preferences *p = &si->prefs;
@@ -679,6 +681,10 @@ describe_dead_child (saver_info *si, pid_t kid, int wait_status)
 		   blurb(), screen_no, (unsigned long) kid, name,
 		   signal_name (WTERMSIG(wait_status)));
            */
+# ifdef LOG_CPU_TIME
+          long u_ms = rus.ru_utime.tv_usec / 1000 + rus.ru_utime.tv_sec * 1000;
+          long s_ms = rus.ru_stime.tv_usec / 1000 + rus.ru_stime.tv_sec * 1000;
+# endif /* LOG_CPU_TIME */
           write_string (STDERR_FILENO, blurb());
           write_string (STDERR_FILENO, ": ");
           write_long   (STDERR_FILENO, (long) screen_no);
@@ -689,6 +695,20 @@ describe_dead_child (saver_info *si, pid_t kid, int wait_status)
           write_string (STDERR_FILENO, ") terminated with signal ");
           write_long   (STDERR_FILENO, WTERMSIG(wait_status));
           write_string (STDERR_FILENO, ".\n");
+# ifdef LOG_CPU_TIME
+          write_string (STDERR_FILENO, blurb());
+          write_string (STDERR_FILENO, ": ");
+          write_long   (STDERR_FILENO, (long) screen_no);
+          write_string (STDERR_FILENO, ": CPU used: ");
+          write_long   (STDERR_FILENO, (u_ms / 1000));   /* msec -> sec */
+          write_string (STDERR_FILENO, ".");
+          write_long   (STDERR_FILENO, (u_ms % 1000) / 100);
+          write_string (STDERR_FILENO, "u, ");
+          write_long   (STDERR_FILENO, (s_ms / 1000));   /* msec -> sec */
+          write_string (STDERR_FILENO, ".");
+          write_long   (STDERR_FILENO, (s_ms % 1000) / 100);
+          write_string (STDERR_FILENO, "s.\n");
+# endif /* LOG_CPU_TIME */
         }
 
       if (job)
@@ -739,12 +759,18 @@ describe_dead_child (saver_info *si, pid_t kid, int wait_status)
   /* Clear out the pid so that screenhack_running_p() knows it's dead.
    */
   if (!job || job->status == job_dead)
+    {
     for (i = 0; i < si->nscreens; i++)
       {
 	saver_screen_info *ssi = &si->screens[i];
 	if (kid == ssi->pid)
 	  ssi->pid = 0;
       }
+# ifdef HAVE_LIBSYSTEMD
+    if (kid == si->systemd_pid)
+      si->systemd_pid = 0;
+# endif
+    }
 }
 
 #else  /* VMS */
@@ -873,7 +899,12 @@ print_path_error (const char *program)
 pid_t
 fork_and_exec (saver_screen_info *ssi, const char *command)
 {
-  saver_info *si = ssi->global;
+  return fork_and_exec_1 (ssi->global, ssi, command);
+}
+
+pid_t
+fork_and_exec_1 (saver_info *si, saver_screen_info *ssi, const char *command)
+{
   saver_preferences *p = &si->prefs;
   pid_t forked;
 
@@ -890,11 +921,12 @@ fork_and_exec (saver_screen_info *ssi, const char *command)
     case 0:
       close (ConnectionNumber (si->dpy));	/* close display fd */
       limit_subproc_memory (p->inferior_memory_limit, p->verbose_p);
-      hack_subproc_environment (ssi->screen, ssi->screensaver_window);
+      if (ssi)
+        hack_subproc_environment (ssi->screen, ssi->screensaver_window);
 
       if (p->verbose_p)
         fprintf (stderr, "%s: %d: spawning \"%s\" in pid %lu.\n",
-                 blurb(), ssi->number, command,
+                 blurb(), (ssi ? ssi->number : 0), command,
                  (unsigned long) getpid ());
 
       exec_command (p->shell, command, p->nice_inferior);
@@ -909,7 +941,7 @@ fork_and_exec (saver_screen_info *ssi, const char *command)
       break;
 
     default:	/* parent */
-      (void) make_job (forked, ssi->number, command);
+      (void) make_job (forked, (ssi ? ssi->number : 0), command);
       break;
     }
 
@@ -1303,6 +1335,7 @@ get_best_gl_visual (saver_info *si, Screen *screen)
       {
         int result = 0;
         int wait_status = 0;
+        pid_t pid = -1;
 
         FILE *f = fdopen (in, "r");
         unsigned long v = 0;
@@ -1321,8 +1354,17 @@ get_best_gl_visual (saver_info *si, Screen *screen)
             close (errin);
           }
 
-        /* Wait for the child to die. */
-        waitpid (-1, &wait_status, 0);
+        /* Wait for the child to die - wait for this pid only, not others. */
+        pid = waitpid (forked, &wait_status, 0);
+        if (si->prefs.debug_p)
+          {
+            write_string (STDERR_FILENO, blurb());
+            write_string (STDERR_FILENO, ": waitpid(");
+            write_long   (STDERR_FILENO, (long) forked);
+            write_string (STDERR_FILENO, ") ==> ");
+            write_long   (STDERR_FILENO, (long) pid);
+            write_string (STDERR_FILENO, "\n");
+          }
 
         unblock_sigchld();   /* child is dead and waited, unblock now. */
 
@@ -1387,9 +1429,25 @@ save_argv (int argc, char **argv)
 void
 restart_process (saver_info *si)
 {
-  fflush (stdout);
-  fflush (stderr);
-  shutdown_stderr (si);
+  if (si->screen_blanked_p)
+    {
+      unblank_screen (si);
+      XSync (si->dpy, False);
+    }
+
+  emergency_kill_subproc (si);
+
+# ifdef HAVE_LIBSYSTEMD
+  if (si->systemd_pid)  /* Kill background xscreensaver-systemd process */
+    {
+      /* We're exiting, so there's no need to do a full kill_job() here,
+         which would waitpid(). */
+      /* kill_job (si, si->systemd_pid, SIGTERM); */
+      kill (si->systemd_pid, SIGTERM);
+      si->systemd_pid = 0;
+    }
+# endif
+
   if (si->prefs.verbose_p)
     {
       int i;
@@ -1397,12 +1455,13 @@ restart_process (saver_info *si)
       for (i = 0; saved_argv[i]; i++)
 	fprintf (stderr, " %s", saved_argv[i]);
       fprintf (stderr, "\n");
-    }
-  describe_uids (si, stderr);
-  fprintf (stderr, "\n");
 
-  fflush (stdout);
-  fflush (stderr);
+      describe_uids (si, stderr);
+      fprintf (stderr, "\n");
+    }
+
+  shutdown_stderr (si);
+
   execvp (saved_argv [0], saved_argv);	/* shouldn't return */
   {
     char buf [512];
