@@ -1,5 +1,5 @@
 /* windows.c --- turning the screen black; dealing with visuals, virtual roots.
- * xscreensaver, Copyright (c) 1991-2011 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright (c) 1991-2020 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -62,7 +62,7 @@ typedef long PROP32;
 #undef XtPointer
 #define XtAppContext void*
 #define XrmDatabase  void*
-#define XtIntervalId void*
+#define XtIntervalId unsigned long
 #define XtPointer    void*
 #define Widget       void*
 
@@ -74,10 +74,10 @@ typedef long PROP32;
 extern int kill (pid_t, int);		/* signal() is in sys/signal.h... */
 
 Atom XA_VROOT, XA_XSETROOT_ID, XA_ESETROOT_PMAP_ID, XA_XROOTPMAP_ID;
+Atom XA_NET_WM_USER_TIME;
 Atom XA_SCREENSAVER, XA_SCREENSAVER_VERSION, XA_SCREENSAVER_ID;
 Atom XA_SCREENSAVER_STATUS;
 
-
 extern saver_info *global_si_kludge;	/* I hate C so much... */
 
 static void maybe_transfer_grabs (saver_screen_info *ssi,
@@ -396,10 +396,10 @@ ensure_no_screensaver_running (Display *dpy, Screen *screen)
 	  && type != None)
 	{
 	  unsigned char *id;
-	  if (!XGetWindowProperty (dpy, kids[i], XA_SCREENSAVER_ID, 0, 512,
+	  if (XGetWindowProperty (dpy, kids[i], XA_SCREENSAVER_ID, 0, 512,
 				   False, XA_STRING, &type, &format, &nitems,
 				   &bytesafter, &id)
-	      == Success
+	      != Success
 	      || type == None)
 	    id = (unsigned char *) "???";
 
@@ -747,14 +747,37 @@ static RETSIGTYPE
 restore_real_vroot_handler (int sig)
 {
   saver_info *si = global_si_kludge;	/* I hate C so much... */
+  Bool restored_p;
 
   signal (sig, SIG_DFL);
-  if (restore_real_vroot (si))
-    fprintf (real_stderr, "\n%s: %s intercepted, vroot restored.\n",
-	     blurb(), signal_name(sig));
+
+# ifdef HAVE_LIBSYSTEMD
+  if (si->systemd_pid)  /* Kill background xscreensaver-systemd process */
+    {
+      /* We're exiting, so there's no need to do a full kill_job() here,
+         which would waitpid(). */
+      /* kill_job (si, si->systemd_pid, SIGTERM); */
+      kill (si->systemd_pid, SIGTERM);
+      si->systemd_pid = 0;
+    }
+# endif
+
+  /* This is a signal handler, and the following might do X protocol,
+     which is horrible, but we're about to terminate so we don't have
+     a choice... */
+  restored_p = restore_real_vroot (si);
+
+  /* Calling fprintf from a signal handler is also risky, but we're
+     so far into the weeds on that front already... */
+  fprintf (stderr, "\n%s: %s%s\n\n", blurb(), signal_name(sig),
+           (restored_p ? ", vroot restored" : ""));
+
+  shutdown_stderr (si);
+
   kill (getpid (), sig);
 }
 
+
 static void
 catch_signal (saver_info *si, int sig, RETSIGTYPE (*handler) (int))
 {
@@ -837,15 +860,6 @@ saver_sighup_handler (int sig)
   fprintf (stderr, "%s: %s received: restarting...\n",
            blurb(), signal_name(sig));
 
-  if (si->screen_blanked_p)
-    {
-      int i;
-      for (i = 0; i < si->nscreens; i++)
-        kill_screenhack (&si->screens[i]);
-      unblank_screen (si);
-      XSync (si->dpy, False);
-    }
-
   restart_process (si);   /* Does not return */
   abort ();
 }
@@ -861,23 +875,25 @@ saver_exit (saver_info *si, int status, const char *dump_core_reason)
   Bool vrs;
 
   if (exiting)
-    exit(status);
+    exit (status);
 
   exiting = True;
   
   vrs = restore_real_vroot (si);
-  emergency_kill_subproc (si);
-  shutdown_stderr (si);
-
   if (p->verbose_p && vrs)
     fprintf (real_stderr, "%s: old vroot restored.\n", blurb());
 
-  fflush(real_stdout);
+  emergency_kill_subproc (si);
 
-#ifdef VMS	/* on VMS, 1 is the "normal" exit code instead of 0. */
-  if (status == 0) status = 1;
-  else if (status == 1) status = -1;
-#endif
+# ifdef HAVE_LIBSYSTEMD
+  if (si->systemd_pid)  /* Kill background xscreensaver-systemd process */
+    {
+      kill_job (si, si->systemd_pid, SIGTERM);
+      si->systemd_pid = 0;
+    }
+# endif
+
+  shutdown_stderr (si);
 
   bugp = !!dump_core_reason;
 
@@ -897,7 +913,7 @@ saver_exit (saver_info *si, int status, const char *dump_core_reason)
 
       if (bugp)
 	fprintf(real_stderr,
-		"%s: see http://www.jwz.org/xscreensaver/bugs.html\n"
+		"%s: see https://www.jwz.org/xscreensaver/bugs.html\n"
 		"\t\t\tfor bug reporting information.\n\n",
 		blurb());
 
@@ -915,6 +931,11 @@ saver_exit (saver_info *si, int status, const char *dump_core_reason)
       abort();
     }
 
+# ifdef VMS	/* on VMS, 1 is the "normal" exit code instead of 0. */
+  if (status == 0) status = 1;
+  else if (status == 1) status = -1;
+# endif
+
   exit (status);
 }
 
@@ -944,6 +965,9 @@ store_saver_id (saver_screen_info *ssi)
   struct passwd *p = getpwuid (getuid ());
   const char *name, *host;
   char *id;
+# if defined(HAVE_UNAME)
+  struct utsname uts;
+# endif /* UNAME */
 
   /* First store the name and class on the window.
    */
@@ -976,7 +1000,6 @@ store_saver_id (saver_screen_info *ssi)
 
 # if defined(HAVE_UNAME)
   {
-    struct utsname uts;
     if (uname (&uts) < 0)
       host = "???";
     else
@@ -1008,7 +1031,7 @@ store_saver_status (saver_info *si)
 
   status = (PROP32 *) calloc (size, sizeof(PROP32));
 
-  status[0] = (PROP32) (si->screen_blanked_p
+  status[0] = (PROP32) (si->screen_blanked_p || si->locked_p
                         ? (si->locked_p ? XA_LOCK : XA_BLANK)
                         : 0);
   status[1] = (PROP32) si->blank_time;
@@ -1275,7 +1298,11 @@ initialize_screensaver_window_1 (saver_screen_info *ssi)
 	 always exist (though the ID is constant.)  So to use this
 	 window, we'd have to reimplement the ACTIVATE ClientMessage to
 	 tell the *server* to tell *us* to turn on, to cause the window
-	 to get created at the right time.  Gag.  */
+	 to get created at the right time.  Gag.
+
+         See the very long comment at the top of xscreensaver.c for why the
+         MIT-SCREEN-SAVER extension is garbage and should not be used.
+       */
       XScreenSaverSetAttributes (si->dpy, root,
 				 0, 0, width, height, 0,
 				 current_depth, InputOutput, visual,
@@ -1443,6 +1470,7 @@ resize_screensaver_window (saver_info *si)
           if (ssi->cmap)
             XInstallColormap (si->dpy, ssi->cmap);
           XMapRaised (si->dpy, ssi->screensaver_window);
+          XSync (si->dpy, False);
           if (! ssi->pid)
             spawn_screenhack (ssi);
 
@@ -1566,6 +1594,7 @@ raise_window (saver_info *si,
 	  if (!dont_clear || ssi->stderr_overlay_window)
 	    clear_stderr (ssi);
 	  XMapRaised (si->dpy, ssi->screensaver_window);
+          XSync (si->dpy, False);
 #ifdef HAVE_MIT_SAVER_EXTENSION
 	  if (ssi->server_mit_saver_window &&
 	      window_exists_p (si->dpy, ssi->server_mit_saver_window))
@@ -1681,7 +1710,7 @@ blank_screen (saver_info *si)
   raise_window (si, False, False, False);
 
   si->screen_blanked_p = True;
-  si->blank_time = time ((time_t) 0);
+  si->blank_time = time ((time_t *) 0);
   si->last_wall_clock_time = 0;
 
   store_saver_status (si);  /* store blank time */
@@ -1772,15 +1801,18 @@ unblank_screen (saver_info *si)
   {
     Window focus = 0;
     int revert_to;
+    XSync (si->dpy, False);
     XGetInputFocus (si->dpy, &focus, &revert_to);
     if (focus && focus != PointerRoot && focus != None)
       {
+	XErrorHandler old_handler = XSetErrorHandler (BadWindow_ehandler);
 	XWindowAttributes xgwa;
 	xgwa.colormap = 0;
-	XGetWindowAttributes (si->dpy, focus, &xgwa);
-	if (xgwa.colormap &&
+	if (XGetWindowAttributes (si->dpy, focus, &xgwa) &&
+	    xgwa.colormap &&
 	    xgwa.colormap != DefaultColormapOfScreen (xgwa.screen))
 	  XInstallColormap (si->dpy, xgwa.colormap);
+	XSetErrorHandler (old_handler);
       }
   }
 
@@ -1803,7 +1835,7 @@ unblank_screen (saver_info *si)
     XUnmapWindow (si->dpy, si->screens[i].screensaver_window);
 
   si->screen_blanked_p = False;
-  si->blank_time = time ((time_t) 0);
+  si->blank_time = time ((time_t *) 0);
   si->last_wall_clock_time = 0;
 
   store_saver_status (si);  /* store unblank time */
