@@ -1,4 +1,4 @@
-/* xscreensaver-command, Copyright (c) 1991-2013 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver-command, Copyright (c) 1991-2020 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -51,8 +51,8 @@ char *progname;
 Atom XA_VROOT;
 Atom XA_SCREENSAVER, XA_SCREENSAVER_VERSION, XA_SCREENSAVER_RESPONSE;
 Atom XA_SCREENSAVER_ID, XA_SCREENSAVER_STATUS, XA_SELECT, XA_DEMO, XA_EXIT;
-Atom XA_BLANK, XA_LOCK;
-static Atom XA_ACTIVATE, XA_DEACTIVATE, XA_CYCLE, XA_NEXT, XA_PREV;
+Atom XA_BLANK, XA_LOCK, XA_ACTIVATE, XA_SUSPEND, XA_NEXT, XA_PREV;
+static Atom XA_DEACTIVATE, XA_CYCLE;
 static Atom XA_RESTART, XA_PREFS, XA_THROTTLE, XA_UNTHROTTLE;
 
 static char *screensaver_version;
@@ -71,6 +71,9 @@ usage: %s -<option>\n\
   You control a running xscreensaver process by sending it messages\n\
   with this program, xscreensaver-command.  See the man pages for\n\
   details.  These are the arguments understood by xscreensaver-command:\n\
+\n\
+  -quiet        Only print output if an error occurs.\n\
+  -verbose      Opposite of -quiet. Default.\n\
 \n\
   -demo         Ask the xscreensaver process to enter interactive demo mode.\n\
 \n\
@@ -116,6 +119,11 @@ usage: %s -<option>\n\
                 immediately.  This is like -activate, but forces locking as\n\
                 well, even if locking is not the default.  If the saver is\n\
                 already active, this causes it to be locked as well.\n\
+\n\
+  -suspend      Like -activate, but ignores the lock-timeout, if any, and\n\
+                immediately powers off the screen without fading out.\n\
+                This is intended to be run just after your laptop's lid\n\
+                is closed, and just before the CPU halts.\n\
 \n\
   -version      Prints the version of xscreensaver that is currently running\n\
                 on the display -- that is, the actual version number of the\n\
@@ -130,11 +138,11 @@ usage: %s -<option>\n\
   -watch        Prints a line each time the screensaver changes state: when\n\
                 the screen blanks, locks, unblanks, or when the running hack\n\
                 is changed.  This option never returns; it is intended for\n\
-                by shell scripts that want to react to the screensaver in\n\
-                some way.\n\
+                use by shell scripts that want to react to the screensaver\n\
+                in some way.\n\
 \n\
   See the man page for more details.\n\
-  For updates, check http://www.jwz.org/xscreensaver/\n\
+  For updates, check https://www.jwz.org/xscreensaver/\n\
 \n";
 
 /* Note: The "-throttle" command is deprecated -- it predates the XDPMS
@@ -161,6 +169,7 @@ main (int argc, char **argv)
   char *s;
   Atom XA_WATCH = 0;  /* kludge: not really an atom */
   char year[5];
+  Bool verbose_p = TRUE;
 
   progname = argv[0];
   s = strrchr (progname, '/');
@@ -187,6 +196,7 @@ main (int argc, char **argv)
       else if (cmd) USAGE();
       else if (!strncmp (s, "-activate", L))   cmd = &XA_ACTIVATE;
       else if (!strncmp (s, "-deactivate", L)) cmd = &XA_DEACTIVATE;
+      else if (!strncmp (s, "-suspend", L))    cmd = &XA_SUSPEND;
       else if (!strncmp (s, "-cycle", L))      cmd = &XA_CYCLE;
       else if (!strncmp (s, "-next", L))       cmd = &XA_NEXT;
       else if (!strncmp (s, "-prev", L))       cmd = &XA_PREV;
@@ -202,6 +212,8 @@ main (int argc, char **argv)
       else if (!strncmp (s, "-version", L))    cmd = &XA_SCREENSAVER_VERSION;
       else if (!strncmp (s, "-time", L))       cmd = &XA_SCREENSAVER_STATUS;
       else if (!strncmp (s, "-watch", L))      cmd = &XA_WATCH;
+      else if (!strncmp (s, "-quiet", L))      verbose_p = FALSE;
+      else if (!strncmp (s, "-verbose", L))    verbose_p = TRUE;
       else USAGE ();
 
       if (cmd == &XA_SELECT || cmd == &XA_DEMO)
@@ -300,6 +312,7 @@ main (int argc, char **argv)
   XA_SCREENSAVER_RESPONSE = XInternAtom (dpy, "_SCREENSAVER_RESPONSE", False);
   XA_ACTIVATE = XInternAtom (dpy, "ACTIVATE", False);
   XA_DEACTIVATE = XInternAtom (dpy, "DEACTIVATE", False);
+  XA_SUSPEND = XInternAtom (dpy, "SUSPEND", False);
   XA_RESTART = XInternAtom (dpy, "RESTART", False);
   XA_CYCLE = XInternAtom (dpy, "CYCLE", False);
   XA_NEXT = XInternAtom (dpy, "NEXT", False);
@@ -321,13 +334,17 @@ main (int argc, char **argv)
       exit (i);
     }
 
-  if (*cmd == XA_ACTIVATE || *cmd == XA_LOCK ||
+  if (*cmd == XA_ACTIVATE || *cmd == XA_LOCK || *cmd == XA_SUSPEND || 
       *cmd == XA_NEXT || *cmd == XA_PREV || *cmd == XA_SELECT)
     /* People never guess that KeyRelease deactivates the screen saver too,
-       so if we're issuing an activation command, wait a second. */
-    sleep (1);
+       so if we're issuing an activation command, wait a second.
+       No need to do this if stdin is not a tty, meaning we're not being
+       run from the command line.
+     */
+    if (isatty(0))
+      sleep (1);
 
-  i = xscreensaver_command (dpy, *cmd, arg, True, NULL);
+  i = xscreensaver_command (dpy, *cmd, arg, verbose_p, NULL);
   if (i < 0) exit (i);
   else exit (0);
 }
