@@ -87,7 +87,7 @@ scale_image (Display *dpy, Window window, XImage *in,
   int x, y, i;
   unsigned int rpos=0, gpos=0, bpos=0; /* bitfield positions */
   unsigned int rsiz=0, gsiz=0, bsiz=0;
-  unsigned int rmsk=0, gmsk=0, bmsk=0;
+  unsigned long rmsk=0, gmsk=0, bmsk=0;
   unsigned char spread_map[3][256];
   XWindowAttributes xgwa;
   XColor *colors = 0;
@@ -112,9 +112,7 @@ scale_image (Display *dpy, Window window, XImage *in,
     }
   else
     {
-      rmsk = xgwa.visual->red_mask;
-      gmsk = xgwa.visual->green_mask;
-      bmsk = xgwa.visual->blue_mask;
+      visual_rgb_masks (xgwa.screen, xgwa.visual, &rmsk, &gmsk, &bmsk);
       decode_mask (rmsk, &rpos, &rsiz);
       decode_mask (gmsk, &gpos, &gsiz);
       decode_mask (bmsk, &bpos, &bsiz);
@@ -815,7 +813,7 @@ struct terminal_controller_data {
 
 /* The structure of closure linkage throughout this code is so amazingly
    baroque that I can't get to the 'struct state' from where I need it. */
-static const char *global_program;
+static char *global_program;
 static Bool global_fast_p;
 
 
@@ -1827,8 +1825,10 @@ apple2_init (Display *dpy, Window window)
     }
   if (s) free (s);
 
-  global_program = get_string_resource (dpy, "program", "Program");
-  global_fast_p = get_boolean_resource (dpy, "fast", "Boolean");
+  if (!global_program) {
+    global_program = get_string_resource (dpy, "program", "Program");
+    global_fast_p = get_boolean_resource (dpy, "fast", "Boolean");
+  }
 
 
   /* Kludge for MacOS standalone mode: see OSX/SaverRunner.m. */
@@ -1838,7 +1838,8 @@ apple2_init (Display *dpy, Window window)
       {
         st->controller = terminal_controller;
         st->random_p   = False;
-        global_program = getenv ("SHELL");
+        if (global_program) free (global_program);
+        global_program = strdup (getenv ("SHELL"));
         global_fast_p  = True;
       }
   }
@@ -1908,6 +1909,8 @@ apple2_free (Display *dpy, Window window, void *closure)
     if (apple2_one_frame (st->sim))
       abort();  /* should have freed! */
   }
+  if (global_program) free (global_program);
+  global_program = 0;
   free (st);
 }
 
