@@ -29,7 +29,14 @@
 #include <assert.h>
 #include <ctype.h>
 #include <math.h>
-#include <inttypes.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+#if defined(HAVE_STDINT_H)
+# include <stdint.h>
+#elif defined(HAVE_INTTYPES_H)
+# include <inttypes.h>
+#endif
 #include <unistd.h>
 
 #include "yarandom.h"
@@ -112,9 +119,8 @@ static void *ecalloc(uint32_t nelm, size_t nsize){
 
 /* estrdup() - Allocates memory for a new string a returns a copy of the source sting in it. */
 static char *estrdup(const char *source){
-  int ln = strlen(source) + 1;
-  char *s = ecalloc(ln, sizeof(char));
-  strncpy(s,source,ln);
+  char *s = strdup (source);
+  if (!s) abort();
   return s;
 }
 
@@ -542,7 +548,7 @@ static void jmpBIT(machine_6502 *machine, m6502_AddrMode adm){
   Pointer ptr;
   BOOL isValue = getValue(machine, adm, &ptr);
   warnValue(isValue);
-  machine->regP = setBit(machine->regP, ZERO_FL, (ptr.value & machine->regA));
+  machine->regP = setBit(machine->regP, ZERO_FL, !(ptr.value & machine->regA));
   machine->regP = setBit(machine->regP, OVERFLOW_FL, bitOn(ptr.value, OVERFLOW_FL));
   machine->regP = setBit(machine->regP, NEGATIVE_FL, bitOn(ptr.value, NEGATIVE_FL));
   
@@ -1003,8 +1009,8 @@ static void jmpSTY(machine_6502 *machine, m6502_AddrMode adm){
 static void assignOpCodes(m6502_Opcodes *opcodes){
 
  #define SETOP(num, _name, _Imm, _ZP, _ZPX, _ZPY, _ABS, _ABSX, _ABSY, _INDX, _INDY, _SNGL, _BRA, _func) \
-{opcodes[num].name[3] = '\0'; \
- strncpy(opcodes[num].name, _name, 3); opcodes[num].Imm = _Imm; opcodes[num].ZP = _ZP; \
+{sprintf(opcodes[num].name, "%.*s", MAX_LABEL_LEN-1, _name); \
+ opcodes[num].Imm = _Imm; opcodes[num].ZP = _ZP; \
  opcodes[num].ZPX = _ZPX; opcodes[num].ZPY = _ZPY; opcodes[num].ABS = _ABS; \
  opcodes[num].ABSX = _ABSX; opcodes[num].ABSY = _ABSY; opcodes[num].INDX = _INDX; \
  opcodes[num].INDY = _INDY; opcodes[num].SNGL = _SNGL; opcodes[num].BRA = _BRA; \
@@ -1406,9 +1412,7 @@ static BOOL immediate(char **s, Param *param){
     param->type = (**s == '<') ? IMMEDIATE_LESS : IMMEDIATE_GREAT;
     (*s)++; /* move past < or > */
     if (paramLabel(s, &label)){
-      int ln = strlen(label) + 1;
-      strncpy(param->label, label, ln);
-      free(label);
+      sprintf(param->label, "%.*s", MAX_LABEL_LEN-1, label);
       return TRUE;
     }    
     free(label);
@@ -1874,7 +1878,7 @@ static BOOL translate(m6502_Opcodes *op,Param *param, machine_6502 *machine){
 	if (op->BRA) {
 	  pushByte(machine, op->BRA);
           {
-            int diff = (param->lbladdr - machine->defaultCodePC);
+            int diff = abs((int)param->lbladdr - (int)machine->defaultCodePC);
             int backward = (param->lbladdr < machine->defaultCodePC);
             pushByte(machine, (backward) ? 0xff - diff : diff - 1);
           }
@@ -2035,7 +2039,7 @@ static BOOL compileCode(machine_6502 *machine, const char *code){
       AsmLine *p;
       if(asmlist != NULL){
         for (p = asmlist; p != NULL; p = p->next)
-          fprintf(stderr,"%s lbl: %s addr: %d ParamLbl: %s ParamAddr: %d\n",
+          fprintf(stderr,"%s lbl: %s addr: %x ParamLbl: %s ParamAddr: %x\n",
                   p->command, p->label->label, p->label->addr,
                   p->param->label, p->param->lbladdr);
             }
@@ -2062,7 +2066,7 @@ static BOOL compileCode(machine_6502 *machine, const char *code){
     }
   }
   else{
-    fprintf(stderr,"An error occured while parsing the file.\n");  
+    fprintf(stderr,"An error occurred while parsing the file.\n");
     codeOk = FALSE;
   }
   freeallAsmLine(asmlist);
@@ -2110,7 +2114,6 @@ machine_6502 *m6502_build(void){
 
 void m6502_destroy6502(machine_6502 *machine){
   free(machine);
-  machine = NULL;
 }
 
 void m6502_trace(machine_6502 *machine, FILE *output){
@@ -2192,7 +2195,7 @@ void m6502_eval_file(machine_6502 *machine, const char *filename, m6502_Plotter
   do{
     sleep(0); /* XXX */
 #if 0
-    trace(machine);
+    m6502_trace(machine, stdout);
 #endif
     execute(machine);
   }while(machine->codeRunning);
