@@ -14,6 +14,7 @@
  */
 
 #include <math.h>
+#include <time.h>
 #include "screenhack.h"
 
 /* non-user-modifiable immutable definitions */
@@ -100,14 +101,17 @@ static const char *words[] =
   "abstraction",
   "acid",
   "addiction",
+  "affluenza",
   "alertness",
   "Algeria",
+  "antifa",
   "anxiety",
   "aorta",
   "argyle socks",
   "attrition",
   "axis of evil",
   "bamboo",
+  "banana slug",
   "bangle",
   "bankruptcy",
   "baptism",
@@ -115,27 +119,36 @@ static const char *words[] =
   "bellicosity",
   "bells",
   "belly",
+  "bird flu",
   "bliss",
   "bogosity",
   "boobies",
   "boobs",
   "booty",
   "bread",
+  "brogrammers",
   "bubba",
   "burrito",
   "California",
+  "cancer",
   "capybara",
   "cardinality",
   "caribou",
   "carnage",
   "children",
   "chocolate",
+  "chupacabra",
   "CLONE",
   "cock",
+  "congress",
   "constriction",
   "contrition",
   "cop",
   "corpse",
+# ifndef HAVE_IPHONE
+  "coronavirus",
+  "covid-19",
+# endif
   "cowboy",
   "crabapple",
   "craziness",
@@ -152,10 +165,10 @@ static const char *words[] =
   "despair",
   "desperation",
   "disease",
-  "disease",
+  "DNA Lounge",
   "doberman",
   "DOOM",
-  "dreams",
+  "dot com",
   "dreams",
   "drugs",
   "easy",
@@ -174,9 +187,10 @@ static const char *words[] =
   "flatulence",
   "fluff",
   "fnord",
+  "followers",
+  "frak",
   "freedom",
   "fruit",
-  "fruit",
   "futility",
   "gerbils",
   "GOD",
@@ -197,8 +211,12 @@ static const char *words[] =
   "icepick",
   "identity",
   "ignorance",
+  "illuminati",
   "importance",
   "individuality",
+  "influence",
+  "influencers",
+  "influenza",
   "inkling",
   "insurrection",
   "intoxicant",
@@ -214,6 +232,7 @@ static const char *words[] =
   "lattice",
   "lawyer",
   "lemming",
+  "likes",
   "liquidation",
   "lobbyist",
   "love",
@@ -223,9 +242,11 @@ static const char *words[] =
   "malfunction",
   "marmot",
   "marshmallow",
+  "measles",
   "merit",
   "merkin",
   "mescaline",
+  "methane",
   "milk",
   "mischief",
   "mistrust",
@@ -280,7 +301,12 @@ static const char *words[] =
   "respect",
   "revolution",
   "roadrunner",
+  "rootkit",
   "rule",
+  "SARS",
+# ifndef HAVE_IPHONE
+  "SARS-CoV-2",
+# endif
   "savor",
   "scab",
   "scalar",
@@ -289,6 +315,7 @@ static const char *words[] =
   "security",
   "sediment",
   "self worth",
+  "shadow profile",
   "sickness",
   "silicone",
   "slack",
@@ -315,17 +342,21 @@ static const char *words[] =
   "terrorism",
   "terrorist",
   "the impossible",
+  "the panopticon",
   "the unknown",
   "toast",
   "topography",
   "truism",
+  "truthiness",
   "turgid",
+  "twits",
   "underbrush",
   "underling",
   "unguent",
   "unusual",
   "uplink",
   "urge",
+  "vaccines",
   "valor",
   "variance",
   "vaudeville",
@@ -333,6 +364,7 @@ static const char *words[] =
   "vegetarian",
   "venom",
   "verifiability",
+  "very fine people",
   "viagra",
   "vibrator",
   "victim",
@@ -350,7 +382,6 @@ static const char *words[] =
   "words",
   "worm",
   "worship",
-  "worship",
   "Xanax",
   "Xerxes",
   "Xhosa",
@@ -390,14 +421,12 @@ static void bitmapClear (Bitmap *b)
     memset (b->buf, 0, b->widthBytes * b->height);
 }
 
-#if 0
 /* free a bitmap */
 static void bitmapFree (Bitmap *b)
 {
     free (b->buf);
     free (b);
 }
-#endif
 
 
 /* get the byte value at the given byte-offset coordinates in the given
@@ -824,7 +853,7 @@ static void drawDigitChar (struct state *st, Bitmap *b, int x, int y, char c)
     if ((c < '0') || (c > '9'))
       c = '0';
 
-    bitmapDrawChar5x8 (b, x, y, c);
+  bitmapDrawChar5x8 (b, x, y, c);
 }
 
 /* draw a upc/ean digit at the given coordinates */
@@ -1634,6 +1663,12 @@ static void scrollModel (struct state *st)
 	     st->barcodes[st->barcode_count - 1].mag * BARCODE_WIDTH);
 	barcode->x += RAND_FLOAT_01 * 100;
 	barcode->mag = RAND_FLOAT_01 * MAX_MAG;
+
+        if (st->windowWidth < 100 || st->windowHeight < 100) {
+          barcode->mag *= 0.5;
+          if (barcode->mag <= 0) barcode->mag = 1;
+        }
+
 	barcode->y =
 	    RAND_FLOAT_01 * (st->windowHeight - BARCODE_HEIGHT * barcode->mag);
 	if (barcode->y < 0) 
@@ -1728,7 +1763,7 @@ static void updateGrid (struct state *st)
                 s[j] = (random() % 10) + '0';
               s[j++] = '?';
               s[j++] = ':';
-              s[j++] = 0;
+              s[j] = 0;
             }
 
           /* change one digit in this barcode */
@@ -1898,11 +1933,24 @@ static void
 barcode_reshape (Display *dpy, Window window, void *closure, 
                  unsigned int w, unsigned int h)
 {
+  struct state *st = (struct state *) closure;
+  st->windowWidth = w;
+  st->windowHeight = h;
 }
 
 static void
 barcode_free (Display *dpy, Window window, void *closure)
 {
+  struct state *st = (struct state *) closure;
+  int i;
+  XFreeGC (dpy, st->theGC);
+  st->theImage->data = 0;
+  XDestroyImage (st->theImage);
+  bitmapFree (st->theBitmap);
+  for (i = 0; i < st->barcode_max; i++)
+    bitmapFree (st->barcodes[i].bitmap);
+  free (st->barcodes);
+  free (st);
 }
 
 
@@ -1911,6 +1959,7 @@ barcode_free (Display *dpy, Window window, void *closure)
 static const char *barcode_defaults [] = {
     ".background:	black",
     ".foreground:	green",
+    ".lowrez:           true",
     "*fpsSolid: 	true",
     "*delay:		10000",
     "*mode:		scroll",
