@@ -223,7 +223,7 @@ self_similar_normalized (struct state *st,
 	double    x = 0.0;
 	double    y = 0.0;
 
-	replacement = (Position*)(malloc (segment_count * sizeof (Segment)));
+	replacement = (Position*)(malloc (segment_count * sizeof (*replacement)));
 	copy_points (segment_count, points, replacement);
 	assert (fabs ((replacement [segment_count - 1].x) - 1.0) < EPSILON);
 	assert (fabs (replacement [segment_count - 1].y) < EPSILON);
@@ -701,7 +701,8 @@ ccurve_init (Display *dpy, Window window)
     st->context = XCreateGC (st->dpy, st->window, GCForeground | GCBackground,
 			 &values);
     st->color_count = MAXIMUM_COLOR_COUNT;
-    make_color_loop (st->dpy, st->color_map,
+    make_color_loop (hack_attributes.screen, hack_attributes.visual,
+                     st->color_map,
 		     0,   1, 1,
 		     120, 1, 1,
 		     240, 1, 1,
@@ -732,6 +733,7 @@ ccurve_draw (Display *dpy, Window window, void *closure)
           {
 	st->draw_segment_count
 	    = lengths [random () % (sizeof (lengths) / sizeof (int))];
+        if (st->draw_segments) free (st->draw_segments);
 	st->draw_segments
 	    = (Segment*)(malloc ((st->draw_segment_count) * sizeof (Segment)));
 	select_pattern (st->draw_segment_count, st->draw_segments);
@@ -796,6 +798,7 @@ ccurve_draw (Display *dpy, Window window, void *closure)
 		       0, 0);
 	}
         st->draw_index++;
+        /* #### mi->recursion_depth = st->draw_index; */
 
         if (st->draw_index >= st->draw_iterations)
           {
@@ -812,17 +815,35 @@ static void
 ccurve_reshape (Display *dpy, Window window, void *closure, 
                  unsigned int w, unsigned int h)
 {
+  struct state *st = (struct state *) closure;
+  XWindowAttributes xgwa;
+  st->width = w;
+  st->height = h;
+  XGetWindowAttributes (st->dpy, st->window, &xgwa);
+  XFreePixmap (dpy, st->pixmap);
+  st->pixmap = XCreatePixmap (st->dpy, st->window, st->width, st->height,
+                              xgwa.depth);
 }
 
 static Bool
 ccurve_event (Display *dpy, Window window, void *closure, XEvent *event)
 {
+  struct state *st = (struct state *) closure;
+  if (screenhack_event_helper (dpy, window, event))
+    {
+      st->draw_index = 0;
+      return True;
+    }
   return False;
 }
 
 static void
 ccurve_free (Display *dpy, Window window, void *closure)
 {
+  struct state *st = (struct state *) closure;
+  XFreeGC (dpy, st->context);
+  if (st->draw_segments) free (st->draw_segments);
+  free (st);
 }
 
 
@@ -833,6 +854,9 @@ static const char *ccurve_defaults [] =
     ".delay:      3",
     ".pause:      0.4",
     ".limit: 200000",
+#ifdef HAVE_MOBILE
+    "*ignoreRotation: True",
+#endif
     0
 };
 
