@@ -41,8 +41,8 @@ struct glb_config glb_config =
 # define DEFAULTS	"*delay:	10000   \n"	\
 			"*showFPS:      False   \n"
 
-# define refresh_bubble3d 0
-# define bubble3d_handle_event 0
+# define release_bubble3d 0
+# define bubble3d_handle_event xlockmore_no_events
 #include "xlockmore.h"
 #else
 #include "xlock.h"
@@ -79,10 +79,10 @@ ModStruct   bubbles3d_description =
 {"bubbles3d",
  "init_bubble3d",
  "draw_bubble3d",
- "release_bubble3d",
+ NULL,
  "change_bubble3d",
  "init_bubble3d",
- NULL,
+ "free_bubble3d",
  &bubble3d_opts,
  1000, 1, 2, 1, 64, 1.0, "",
  "Richard Jones's GL bubbles",
@@ -138,15 +138,21 @@ init(struct context *c)
 }
 
 ENTRYPOINT void
-reshape_bubble3d(ModeInfo *mi, int w, int h)
+reshape_bubble3d(ModeInfo *mi, int width, int height)
 {
-	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-	gluPerspective(45, (GLdouble) w / (GLdouble) h, 3, 8);
-	glMatrixMode(GL_MODELVIEW);
-	glLoadIdentity();
-	glTranslatef(0, 0, -5);
+  double h = (GLfloat) height / (GLfloat) width;  
+
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    h = height / (GLfloat) width;
+  }
+  glViewport(0, 0, width, height);
+  glMatrixMode(GL_PROJECTION);
+  glLoadIdentity();
+  gluPerspective(45, 1/h, 3, 8);
+  glMatrixMode(GL_MODELVIEW);
+  glLoadIdentity();
+  glTranslatef(0, 0, -5);
 }
 
 static void
@@ -163,12 +169,7 @@ init_bubble3d(ModeInfo * mi)
 	int         screen = MI_SCREEN(mi);
 	struct context *c;
 
-	if (contexts == 0) {
-		contexts = (struct context *) calloc(sizeof (struct context), MI_NUM_SCREENS(mi));
-
-		if (contexts == 0)
-			return;
-	}
+	MI_INIT (mi, contexts);
 	c = &contexts[screen];
 	c->glx_context = init_GL(mi);
 	init_colors(mi);
@@ -194,7 +195,7 @@ draw_bubble3d(ModeInfo * mi)
 	if (!c->glx_context)
 		return;
 
-	glXMakeCurrent(display, window, *(c->glx_context));
+	glXMakeCurrent(display, window, *c->glx_context);
 
         glb_config.polygon_count = 0;
         glPushMatrix();
@@ -227,19 +228,12 @@ change_bubble3d(ModeInfo * mi)
 #endif /* !STANDALONE */
 
 ENTRYPOINT void
-release_bubble3d(ModeInfo * mi)
+free_bubble3d(ModeInfo * mi)
 {
-  if (contexts) {
-    int screen;
-    for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-      struct context *c = &contexts[screen];
-      if (c->draw_context)
-        glb_draw_end(c->draw_context);
-    }
-    free (contexts);
-    contexts = 0;
-  }
-  FreeAllGL(mi);
+  struct context *c = &contexts[MI_SCREEN(mi)];
+  if (!c->glx_context) return;
+  glXMakeCurrent (MI_DISPLAY(mi), MI_WINDOW(mi), *c->glx_context);
+  if (c->draw_context) glb_draw_end(c->draw_context);
 }
 
 XSCREENSAVER_MODULE ("Bubble3D", bubble3d)
