@@ -9,14 +9,25 @@
 #ifndef __BUILD_LWO_H__
 #define __BUILD_LWO_H__
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #ifdef STANDALONE
-# ifdef HAVE_COCOA
-#  include <OpenGL/gl.h>
-# else
+# ifndef HAVE_JWXYZ
 #  include <GL/gl.h>
 # endif
 #endif
 
+#ifdef HAVE_ANDROID
+# include <GLES/gl.h>
+#endif
+
+
+#ifdef HAVE_JWZGLES
+# include "jwzgles.h"
+#endif /* HAVE_JWZGLES */
+
 struct lwo {
 	int             num_pnts;
 	const GLfloat  *pnts;
