@@ -20,7 +20,7 @@
                        "*showFPS:   False \n" \
 		       "*wireframe: False \n" \
 
-# define refresh_chess 0
+# define release_chess 0
 # include "xlockmore.h"
 
 #else
@@ -73,7 +73,7 @@ ENTRYPOINT ModeSpecOpt chess_opts = {countof(opts), opts, countof(vars), vars, N
 
 #ifdef USE_MODULES
 ModStruct   chess_description =
-{"chess", "init_chess", "draw_chess", "release_chess",
+{"chess", "init_chess", "draw_chess", NULL,
  "draw_chess", "init_chess", NULL, &chess_opts,
  1000, 1, 2, 1, 4, 1.0, "",
  "Chess", 0, NULL};
@@ -786,7 +786,15 @@ static void display(ModeInfo *mi, Chesscreen *cs)
 ENTRYPOINT void reshape_chess(ModeInfo *mi, int width, int height) 
 {
   GLfloat h = (GLfloat) height / (GLfloat) width;
-  glViewport(0,0, width, height);
+  int y = 0;
+
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport(0,y, width, height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(45, 1/h, 2.0, 30.0);
@@ -799,9 +807,7 @@ ENTRYPOINT void init_chess(ModeInfo *mi)
   Chesscreen *cs;
   int screen = MI_SCREEN(mi);
 
-  if(!qs && 
-     !(qs = (Chesscreen *) calloc(MI_NUM_SCREENS(mi), sizeof(Chesscreen))))
-    return;
+  MI_INIT(mi, qs);
   
   cs = &qs[screen];
   cs->window = MI_WINDOW(mi);
@@ -887,7 +893,7 @@ ENTRYPOINT void draw_chess(ModeInfo *mi)
   if(!cs->glx_context)
     return;
 
-  glXMakeCurrent(disp, w, *(cs->glx_context));
+  glXMakeCurrent(disp, w, *cs->glx_context);
 
   /** code for moving a piece */
   if(cs->moving && ++cs->steps == 100) {
@@ -959,14 +965,20 @@ ENTRYPOINT void draw_chess(ModeInfo *mi)
   glXSwapBuffers(disp, w);
 }
 
-/** bust it */
-ENTRYPOINT void release_chess(ModeInfo *mi) 
-{
-  if(qs)
-    free((void *) qs);
-  qs = 0;
 
-  FreeAllGL(mi);
+ENTRYPOINT void free_chess(ModeInfo *mi) 
+{
+  Chesscreen *cs = &qs[MI_SCREEN(mi)];
+  int i;
+  if (!cs->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *cs->glx_context);
+  gltrackball_free (cs->trackball);
+  if (cs->piecetexture) glDeleteTextures (1, &cs->piecetexture);
+  if (cs->boardtexture) glDeleteTextures (1, &cs->boardtexture);
+
+  /* this is horrible! List numbers are hardcoded! */
+  for (i = 1; i <= 20; i++)
+    if (glIsList(i)) glDeleteLists(i, 1);
 }
 
 XSCREENSAVER_MODULE_2 ("Endgame", endgame, chess)
