@@ -46,7 +46,7 @@
                                         "*useSHM:       True \n" \
 										"*suppressRotationAnimation: True\n" \
 
-# define refresh_gflux 0
+# define release_gflux 0
 # include "xlockmore.h"				/* from the xscreensaver distribution */
 #else /* !STANDALONE */
 # include "xlock.h"					/* from the xlockmore distribution */
@@ -168,7 +168,7 @@ ENTRYPOINT ModeSpecOpt gflux_opts = {countof(opts), opts, countof(vars), vars, d
 
 #ifdef USE_MODULES
 ModStruct   gflux_description =
-{"gflux", "init_gflux", "draw_gflux", "release_gflux",
+{"gflux", "init_gflux", "draw_gflux", NULL,
  "draw_gflux", "init_gflux", NULL, &gflux_opts,
  1000, 1, 2, 1, 4, 1.0, "",
  "GFlux: an OpenGL gflux", 0, NULL};
@@ -274,7 +274,7 @@ ENTRYPOINT void draw_gflux(ModeInfo * mi)
     /* Just keep running before the texture has come in. */
     /* if (gp->waiting_for_image_p) return; */
 
-    glXMakeCurrent(display, window, *(gp->glx_context));
+    glXMakeCurrent(display, window, *gp->glx_context);
 
     calcGrid(gp);
     mi->polygon_count = gp->drawFunc(gp);
@@ -358,11 +358,7 @@ ENTRYPOINT void init_gflux(ModeInfo * mi)
     int screen = MI_SCREEN(mi);
     gfluxstruct *gp;
 
-    if (gfluxes == NULL) {
-        if ((gfluxes = (gfluxstruct *) 
-                 calloc(MI_NUM_SCREENS(mi), sizeof (gfluxstruct))) == NULL)
-            return;
-    }
+    MI_INIT(mi, gfluxes);
     gp = &gfluxes[screen];
 
     gp->trackball = gltrackball_init (True);
@@ -385,6 +381,7 @@ ENTRYPOINT void init_gflux(ModeInfo * mi)
                    progname, s);
           exit (1);
         }
+      if (s) free (s);
     }
 
     gp->modeinfo = mi;
@@ -397,17 +394,15 @@ ENTRYPOINT void init_gflux(ModeInfo * mi)
     }
 }
 
-/* cleanup code */
-ENTRYPOINT void release_gflux(ModeInfo * mi)
+ENTRYPOINT void free_gflux(ModeInfo * mi)
 {
-    if (gfluxes != NULL) {
-      free((void *) gfluxes);
-      gfluxes = NULL;
-    }
-    FreeAllGL(mi);
+  gfluxstruct *gp = &gfluxes[MI_SCREEN(mi)];
+  if (!gp->glx_context) return;
+  glXMakeCurrent (MI_DISPLAY(mi), MI_WINDOW(mi), *gp->glx_context);
+  gltrackball_free (gp->trackball);
+  if (gp->texName) glDeleteTextures (1, &gp->texName);
 }
 
-
 static void createTexture(gfluxstruct *gp)
 {
   int size = 4;
