@@ -28,7 +28,7 @@
 				 "*showFPS:   False\n" \
 				 "*wireframe: False\n"
 				 
-# define refresh_glhanoi 0
+# define release_glhanoi 0
 
 /* polygon resolution of poles and disks */
 #define NSLICE 32
@@ -235,8 +235,8 @@ ENTRYPOINT ModeSpecOpt glhanoi_opts = { countof(opts), opts, countof(vars), vars
 #ifdef USE_MODULES
 
 ModStruct glhanoi_description = {
-	"glhanoi", "init_glhanoi", "draw_glhanoi", "release_glhanoi",
-	"draw_glhanoi", "init_glhanoi", NULL, &glhanoi_opts,
+	"glhanoi", "init_glhanoi", "draw_glhanoi", NULL,
+	"draw_glhanoi", "init_glhanoi", "free_glhanoi", &glhanoi_opts,
 	1000, 1, 2, 1, 4, 1.0, "",
 	"Towers of Hanoi", 0, NULL
 };
@@ -1012,6 +1012,8 @@ static int drawTube(GLdouble bottomRadius, GLdouble topRadius,
 	glVertex3f(0.0, y, innerRadius);
 	glVertex3f(0.0, y, radius);
 	glEnd();
+    free (cosCache);
+    free (sinCache);
     return polys;
 }
 
@@ -1294,7 +1296,7 @@ static void initData(glhcfg *glhanoi)
 		glhanoi->pole[i].size = glhanoi->numberOfDisks;
 	}
 	checkAllocAndExit(
-			!!(glhanoi->diskPos = calloc(glhanoi->numberOfDisks, sizeof(double))),
+			!!(glhanoi->diskPos = calloc(glhanoi->numberOfDisks, sizeof(float))),
 			"diskPos");
 
 	if (glhanoi->trailQSize) {
@@ -1422,13 +1424,14 @@ static GLubyte *makeTexture(glhcfg *glhanoi, int x_size, int y_size, int z_size,
 									   tex_col_t *), tex_col_t * colours)
 {
 	int i, j, k;
-	GLubyte *textureData;
+	GLuint *textureData;
 	GLuint *texturePtr;
 	double x, y, z;
 	double xi, yi, zi;
 
+	/* As we use GL_RGBA format, we must assign 4 bytes per element */
 	if((textureData =
-		calloc(x_size * y_size * z_size, sizeof(GLuint))) == NULL) {
+		calloc(x_size * y_size * z_size, sizeof(*texturePtr))) == NULL) {
 		return NULL;
 	}
 
@@ -1437,7 +1440,7 @@ static GLubyte *makeTexture(glhcfg *glhanoi, int x_size, int y_size, int z_size,
 	zi = 1.0 / z_size;
 
 	z = 0.0;
-	texturePtr = (void *)textureData;
+	texturePtr = textureData;
 	for(k = 0; k < z_size; k++, z += zi) {
 		y = 0.0;
 		for(j = 0; j < y_size; j++, y += yi) {
@@ -1448,7 +1451,7 @@ static GLubyte *makeTexture(glhcfg *glhanoi, int x_size, int y_size, int z_size,
 			}
 		}
 	}
-	return textureData;
+	return (GLubyte *)textureData;
 }
 
 static void freeTexCols(tex_col_t*p)
@@ -1845,11 +1848,23 @@ static int drawTrails(glhcfg *glhanoi) {
  */
 ENTRYPOINT void reshape_glhanoi(ModeInfo * mi, int width, int height)
 {
-	glViewport(0, 0, (GLint) width, (GLint) height);
+	glhcfg *glhanoi = &glhanoi_cfg[MI_SCREEN(mi)];
+    double h = (GLfloat) height / (GLfloat) width;  
+    int y = 0;
+
+    if (width > height * 5) {   /* tiny window: show middle */
+      height = width * 9/16;
+      y = -height/2;
+      h = height / (GLfloat) width;
+    }
+
+	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *glhanoi->glx_context);
+
+	glViewport(0, y, (GLint) width, (GLint) height);
 
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
-	gluPerspective(30.0, (GLdouble) width / (GLdouble) height, 1.0,
+	gluPerspective(30.0, 1/h, 1.0,
 				   2 * MAX_CAMERA_RADIUS);
 
 	glMatrixMode(GL_MODELVIEW);
@@ -1861,11 +1876,7 @@ ENTRYPOINT void reshape_glhanoi(ModeInfo * mi, int width, int height)
 ENTRYPOINT void init_glhanoi(ModeInfo * mi)
 {
 	glhcfg *glhanoi;
-	if(!glhanoi_cfg) {
-		glhanoi_cfg =
-			(glhcfg *) calloc(MI_NUM_SCREENS(mi), sizeof(glhcfg));
-		checkAllocAndExit(!!glhanoi_cfg, "configuration");
-	}
+	MI_INIT(mi, glhanoi_cfg);
 
 	glhanoi = &glhanoi_cfg[MI_SCREEN(mi)];
 	glhanoi->glx_context = init_GL(mi);
@@ -1954,7 +1965,7 @@ ENTRYPOINT void draw_glhanoi(ModeInfo * mi)
 	if(!glhanoi->glx_context)
 		return;
 
-    glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(glhanoi->glx_context));
+    glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *glhanoi->glx_context);
 
 	glPolygonMode(GL_FRONT, glhanoi->wire ? GL_LINE : GL_FILL);
 
@@ -2050,31 +2061,34 @@ ENTRYPOINT Bool glhanoi_handle_event(ModeInfo * mi, XEvent * event)
 	return False;
 }
 
-ENTRYPOINT void release_glhanoi(ModeInfo * mi)
+ENTRYPOINT void free_glhanoi(ModeInfo * mi)
 {
-	if(glhanoi_cfg != NULL) {
-		int screen;
-		for(screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			int i;
-			int j;
-			glhcfg *glh = &glhanoi_cfg[screen];
-			glDeleteLists(glh->floorList, 1);
-			glDeleteLists(glh->baseList, 1);
-			glDeleteLists(glh->poleList, 1);
-                        glDeleteLists(glh->textureNames[0], 2);
-			for(j = 0; j < glh->numberOfDisks; ++j) {
-				glDeleteLists(glh->disk[j].displayList, 1);
-			}
-			free(glh->disk);
-			for(i = 0; i < glh->numberOfPoles; i++) {
-				if(glh->pole[i].data != NULL) {
-					free(glh->pole[i].data);
-				}
-			}
-		}
-	}
-	free(glhanoi_cfg);
-	glhanoi_cfg = NULL;
+	glhcfg *glh = &glhanoi_cfg[MI_SCREEN(mi)];
+	int i;
+	int j;
+
+    if (!glh->glx_context) return;
+	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *glh->glx_context);
+
+    free_rotator (glh->the_rotator);
+    if (glh->pole) {
+      for (i = 0; i < glh->numberOfPoles; i++)
+        if (glh->pole[i].data) free (glh->pole[i].data);
+      free (glh->pole);
+    }
+    if (glh->diskPos) free (glh->diskPos);
+    if (glh->trailQ) free (glh->trailQ);
+    if (glh->solveStack) free (glh->solveStack);
+
+    glDeleteLists(glh->floorList, 1);
+    glDeleteLists(glh->baseList, 1);
+    glDeleteLists(glh->poleList, 1);
+            glDeleteLists(glh->textureNames[0], 2);
+    for(j = 0; j < glh->numberOfDisks; ++j) {
+        glDeleteLists(glh->disk[j].displayList, 1);
+    }
+    free(glh->disk);
+	glDeleteTextures (N_TEXTURES, glh->textureNames);
 }
 
 XSCREENSAVER_MODULE ("GLHanoi", glhanoi)
