@@ -1,10 +1,11 @@
-/* glslideshow, Copyright (c) 2003-2012 Jamie Zawinski <jwz@jwz.org>
+/* glslideshow, Copyright (c) 2003-2020 Jamie Zawinski <jwz@jwz.org>
  * Loads a sequence of images and smoothly pans around them; crossfades
  * when loading new images.
  *
  * Originally written by Mike Oliphant <oliphant@gtk.org> (c) 2002, 2003.
  * Rewritten by jwz, 21-Jun-2003.
  * Rewritten by jwz again, 6-Feb-2005.
+ * Modified by Richard Weeks <rtweeks21@gmail.com> Copyright (c) 2020
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -16,14 +17,6 @@
  *
  *****************************************************************************
  *
- * TODO:
- *
- * - When a new image is loaded, there is a glitch: animation pauses during
- *   the period when we're loading the image-to-fade-in.  On fast (2GHz)
- *   machines, this stutter is short but noticable (usually around 1/10th
- *   second.)  On slower machines, it can be much more pronounced.
- *   This turns out to be hard to fix...
- *
  *   Image loading happens in three stages:
  *
  *    1: Fork a process and run xscreensaver-getimage in the background.
@@ -34,14 +27,11 @@
  *       (or XShmGetImage.)
  *
  *    3: Once we have the bits, we must convert them from server-native bitmap
- *       layout to 32 bit RGBA in client-endianness, to make them usable as
- *       OpenGL textures.
+ *       layout to 32 bit RGBA in client-endianness, and copy that into an
+ *       OpenGL texture.
  *
- *    4: We must actually construct a texture.
- *
- *   So, the speed of step 1 doesn't really matter, since that happens in
- *   the background.  But steps 2, 3, and 4 happen in *this* process, and
- *   cause the visible glitch.
+ *   The speed of step 1 doesn't really matter, since that happens in the
+ *   background.  Steps 2 and 3 happen in *this* process.
  *
  *   Step 2 can't be moved to another process without opening a second
  *   connection to the X server, which is pretty heavy-weight.  (That would
@@ -49,23 +39,13 @@
  *   retrieve the pixmap, and feed it back to us through a pipe or
  *   something.)
  *
- *   Step 3 might be able to be optimized by coding tuned versions of
- *   grab-ximage.c:copy_ximage() for the most common depths and bit orders.
- *   (Or by moving it into the other process along with step 2.)
+ *   Step 3 is carried out over the course of several animation frames.  The
+ *   bits are processed in "stripes" small enough to complete within a single
+ *   frame.  Each stripe is first converted to a client-endian, 32 bit RGBA
+ *   XImage, then copied into the partially completed OpenGL texture.
  *
- *   Step 4 is the hard one, though.  It might be possible to speed up this
- *   step if there is some way to allow two GL processes share texture
- *   data.  Unless, of course, all the time being consumed by step 4 is
- *   because the graphics pipeline is flooded, in which case, that other
- *   process would starve the screen anyway.
- *
- *   Is it possible to use a single GLX context in a multithreaded way?
- *   Or use a second GLX context, but allow the two contexts to share data?
- *   I can't find any documentation about this.
- *
- *   How does Apple do this with their MacOSX slideshow screen saver?
- *   Perhaps it's easier for them because their OpenGL libraries have
- *   thread support at a lower level?
+ *   This entire process is accomplished via a texture loader created with
+ *   alloc_texture_loader.
  */
 
 #define DEFAULTS  "*delay:           20000                \n" \
@@ -73,12 +53,11 @@
                   "*showFPS:         False                \n" \
 	          "*fpsSolid:        True                 \n" \
 	          "*useSHM:          True                 \n" \
-		  "*titleFont:       -*-helvetica-medium-r-normal-*-180-*\n" \
+            "*titleFont: -*-helvetica-medium-r-normal-*-*-180-*-*-*-*-*-*\n" \
                   "*desktopGrabber:  xscreensaver-getimage -no-desktop %s\n" \
 		  "*grabDesktopImages:   False \n" \
 		  "*chooseRandomImages:  True  \n"
 
-# define refresh_slideshow 0
 # define release_slideshow 0
 # include "xlockmore.h"
 
@@ -99,7 +78,7 @@
 # define DEF_MIPMAP         "True"
 
 #include "grab-ximage.h"
-#include "glxfonts.h"
+#include "texfont.h"
 
 typedef struct {
   double x, y, w, h;
@@ -117,6 +96,7 @@ typedef struct {
                                       on screen */
   GLuint texid;			   /* which texture contains the image */
   int refcount;			   /* how many sprites refer to this image */
+  texture_loader_t *loader; /* asynchronous image loader */
 } image;
 
 
@@ -160,12 +140,7 @@ typedef struct {
   Bool checked_fps_p;		/* Whether we have checked for a low
                                    frame rate. */
 
-# ifdef HAVE_GLBITMAP
-  XFontStruct *xfont;		/* for printing image file names */
-  GLuint font_dlist;
-# else
-  texture_font_data *font_data;
-# endif
+  texture_font_data *font_data;	/* for printing image file names */
 
   int sprite_id, image_id;      /* debugging id counters */
 
@@ -231,7 +206,7 @@ ENTRYPOINT ModeSpecOpt slideshow_opts = {countof(opts), opts, countof(vars), var
 static const char *
 blurb (void)
 {
-# ifdef HAVE_COCOA
+# ifdef HAVE_JWXYZ
   return "GLSlideshow";
 # else
   static char buf[255];
@@ -304,6 +279,44 @@ alloc_image (ModeInfo *mi)
 }
 
 
+/* Allocate an image structure and start asynchronous file loading in the
+   background.
+
+   The texture_loader_t referenced by *result->loader must be stepped with
+   step_texture_loader() until it calls back to the callback function passed
+   to it.
+ */
+static image *
+alloc_image_incremental (ModeInfo *mi)
+{
+  slideshow_state *ss = &sss[MI_SCREEN(mi)];
+  int wire = MI_IS_WIREFRAME(mi);
+  image *img = (image *) calloc (1, sizeof (*img));
+
+  img->id = ++ss->image_id;
+  img->loaded_p = False;
+  img->used_p = False;
+  img->mi = mi;
+
+  glGenTextures (1, &img->texid);
+  if (img->texid <= 0) abort();
+
+  ss->image_load_time = ss->now;
+
+  if (wire)
+    image_loaded_cb (0, 0, 0, 0, 0, 0, img);
+  else
+    img->loader = alloc_texture_loader (mi->xgwa.screen, mi->window,
+                                        *ss->glx_context, 0, 0, mipmap_p,
+                                        img->texid);
+
+  ss->images[ss->nimages++] = img;
+  if (ss->nimages >= countof(ss->images)) abort();
+
+  return img;
+}
+
+
 /* Callback that tells us that the texture has been loaded.
  */
 static void
@@ -327,6 +340,13 @@ image_loaded_cb (const char *filename, XRectangle *geom,
       goto DONE;
     }
 
+  if (img->loader)
+    {
+      texture_loader_t *loader = img->loader;
+      img->loader = 0;
+      free_texture_loader(loader);
+    }
+
   if (image_width == 0 || image_height == 0)
     exit (1);
 
@@ -367,11 +387,10 @@ image_loaded_cb (const char *filename, XRectangle *geom,
   */
   if (img->title && img->title[0] == '/')
     {
-      /* strip filename to part between last "/" and last ".". */
+      /* strip filename to part between last "/" and end. */
+      /* xscreensaver-getimage has already stripped off the extension. */
       char *s = strrchr (img->title, '/');
       if (s) strcpy (img->title, s+1);
-      s = strrchr (img->title, '.');
-      if (s) *s = 0;
     }
 
   if (debug_p)
@@ -394,8 +413,8 @@ destroy_image (ModeInfo *mi, image *img)
   int i;
 
   if (!img) abort();
-  if (!img->loaded_p) abort();
-  if (!img->used_p) abort();
+  /* if (!img->loaded_p) abort(); */
+  /* if (!img->used_p) abort(); */
   if (img->texid <= 0) abort();
   if (img->refcount != 0) abort();
 
@@ -463,7 +482,7 @@ get_image (ModeInfo *mi)
   /* Make sure that there is always one unused image in the pipe.
    */
   if (!new_img && !loading_img)
-    alloc_image (mi);
+    alloc_image_incremental (mi);
 
   return img;
 }
@@ -811,29 +830,13 @@ draw_sprite (ModeInfo *mi, sprite *sp)
 
 
     if (do_titles &&
-        img->title && *img->title)
+        img->title && *img->title &&
+        (sp->state == IN || sp->state == FULL))
       {
-        int x = 10;
-        int y = mi->xgwa.height - 10;
-        glColor4f (0, 0, 0, sp->opacity);   /* cheap-assed dropshadow */
-        print_gl_string (mi->dpy,
-# ifdef HAVE_GLBITMAP
-                         ss->xfont, ss->font_dlist,
-# else
-                         ss->font_data,
-# endif
-                         mi->xgwa.width, mi->xgwa.height, x, y,
-                         img->title, False);
-        x++; y++;
         glColor4f (1, 1, 1, sp->opacity);
-        print_gl_string (mi->dpy,
-# ifdef HAVE_GLBITMAP
-                         ss->xfont, ss->font_dlist,
-# else
-                         ss->font_data,
-# endif
-                         mi->xgwa.width, mi->xgwa.height, x, y,
-                         img->title, False);
+        print_texture_label (mi->dpy, ss->font_data,
+                             mi->xgwa.width, mi->xgwa.height,
+                             1, img->title);
       }
   }
   glPopMatrix();
@@ -888,6 +891,7 @@ draw_sprites (ModeInfo *mi)
 
   glPushMatrix();
 
+/*
   {
     GLfloat rot = current_device_rotation();
     glTranslatef (0.5, 0.5, 0);
@@ -900,6 +904,7 @@ draw_sprites (ModeInfo *mi)
       }
     glTranslatef (-0.5, -0.5, 0);
   }
+*/
 
   for (i = 0; i < ss->nsprites; i++)
     draw_sprite (mi, ss->sprites[i]);
@@ -932,6 +937,7 @@ reshape_slideshow (ModeInfo *mi, int width, int height)
   glViewport (0, 0, width, height);
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity();
+  glRotatef (current_device_rotation(), 0, 0, 1);
   glMatrixMode (GL_MODELVIEW);
   glLoadIdentity();
 
@@ -957,24 +963,7 @@ slideshow_handle_event (ModeInfo *mi, XEvent *event)
 {
   slideshow_state *ss = &sss[MI_SCREEN(mi)];
 
-  if (event->xany.type == ButtonPress &&
-      event->xbutton.button == Button1)
-    {
-      ss->change_now_p = True;
-      return True;
-    }
-  else if (event->xany.type == KeyPress)
-    {
-      KeySym keysym;
-      char c = 0;
-      XLookupString (&event->xkey, &c, 1, &keysym, 0);
-      if (c == ' ' || c == '\r' || c == '\n' || c == '\t')
-        {
-          ss->change_now_p = True;
-          return True;
-        }
-    }
-  else if (event->xany.type == Expose ||
+  if (event->xany.type == Expose ||
            event->xany.type == GraphicsExpose ||
            event->xany.type == VisibilityNotify)
     {
@@ -983,6 +972,11 @@ slideshow_handle_event (ModeInfo *mi, XEvent *event)
         fprintf (stderr, "%s: exposure\n", blurb());
       return False;
     }
+  else if (screenhack_event_helper (MI_DISPLAY(mi), MI_WINDOW(mi), event))
+    {
+      ss->change_now_p = True;
+      return True;
+    }
 
   return False;
 }
@@ -1026,7 +1020,7 @@ sanity_check (ModeInfo *mi)
 static void
 check_fps (ModeInfo *mi)
 {
-#ifndef HAVE_COCOA  /* always assume Cocoa is fast enough */
+#ifndef HAVE_JWXYZ  /* always assume Cocoa and mobile are fast enough */
 
   slideshow_state *ss = &sss[MI_SCREEN(mi)];
 
@@ -1078,7 +1072,7 @@ check_fps (ModeInfo *mi)
 
   /* Need this in case zoom changed. */
   reshape_slideshow (mi, mi->xgwa.width, mi->xgwa.height);
-#endif /* HAVE_COCOA */
+#endif /* HAVE_JWXYZ */
 }
 
 
@@ -1098,6 +1092,7 @@ hack_resources (void)
   value.addr = buf2;
   value.size = strlen(buf2);
   XrmPutResource (&db, buf1, "String", &value);
+  free (val);
 #endif
 }
 
@@ -1109,11 +1104,7 @@ init_slideshow (ModeInfo *mi)
   slideshow_state *ss;
   int wire = MI_IS_WIREFRAME(mi);
   
-  if (sss == NULL) {
-    if ((sss = (slideshow_state *)
-         calloc (MI_NUM_SCREENS(mi), sizeof(slideshow_state))) == NULL)
-      return;
-  }
+  MI_INIT (mi, sss);
   ss = &sss[screen];
 
   if ((ss->glx_context = init_GL(mi)) != NULL) {
@@ -1148,11 +1139,7 @@ init_slideshow (ModeInfo *mi)
 
   if (debug_p) glLineWidth (3);
 
-#ifdef HAVE_GLBITMAP
-  load_font (mi->dpy, "titleFont", &ss->xfont, &ss->font_dlist);
-#else
-  ss->font_data = load_texture_font (mi->dpy, "Font");
-#endif
+  ss->font_data = load_texture_font (mi->dpy, "titleFont");
 
   if (debug_p)
     hack_resources();
@@ -1166,6 +1153,10 @@ init_slideshow (ModeInfo *mi)
 }
 
 
+static void
+slideshow_idle (ModeInfo *mi);
+
+
 ENTRYPOINT void
 draw_slideshow (ModeInfo *mi)
 {
@@ -1175,7 +1166,7 @@ draw_slideshow (ModeInfo *mi)
   if (!ss->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(ss->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *ss->glx_context);
 
   if (ss->awaiting_first_image_p)
     {
@@ -1238,7 +1229,14 @@ draw_slideshow (ModeInfo *mi)
     new_sprite (mi);
 
   if (!ss->redisplay_needed_p)
-    return;
+    /* Nothing to do! Don't bother drawing a texture or even swapping the
+       frame buffers. Note that this means that the FPS display will be
+       wrong: "Load" will be frozen on whatever it last was, when in
+       reality it will be close to 0. */
+    {
+      slideshow_idle (mi);
+      return;
+    }
 
   if (debug_p && ss->now - ss->prev_frame_time > 1)
     fprintf (stderr, "%s: static screen for %.1f secs\n",
@@ -1252,9 +1250,62 @@ draw_slideshow (ModeInfo *mi)
   glXSwapBuffers (MI_DISPLAY (mi), MI_WINDOW(mi));
   ss->prev_frame_time = ss->now;
   ss->redisplay_needed_p = False;
+
+  slideshow_idle (mi);
+
   check_fps (mi);
 }
 
+
+ENTRYPOINT void
+free_slideshow (ModeInfo *mi)
+{
+  slideshow_state *ss = &sss[MI_SCREEN(mi)];
+  /* int i; */
+  if (!ss->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *ss->glx_context);
+
+  if (ss->font_data) free_texture_font (ss->font_data);
+  ss->font_data = 0;
+
+# if 0
+  /* The lifetime of these objects is incomprehensible.
+     Doing this causes free pointers to be run from the XtInput.
+   */
+  for (i = ss->nimages-1; i >= 0; i--) {
+    if (ss->images[i] && ss->images[i]->refcount == 0)
+      destroy_image (mi, ss->images[i]);
+  }
+
+  for (i = countof(ss->sprites)-1; i >= 0; i--) {
+    if (ss->sprites[i])
+      destroy_sprite (mi, ss->sprites[i]);
+  }
+# endif
+}
+
+
+static void
+slideshow_idle (ModeInfo *mi)
+{
+  slideshow_state *ss = &sss[MI_SCREEN(mi)];
+  double end_by_time = ss->now + ((double) mi->pause) / 2000000;
+  int i;
+
+  for (i = 0; i < ss->nimages; i++)
+    {
+      image *img = ss->images[i];
+      if (img->loader)
+        {
+          if (texture_loader_failed (img->loader))
+            abort();
+          step_texture_loader (img->loader, end_by_time - double_time(),
+                               image_loaded_cb, img);
+          break; /* only do the first one! */
+        }
+    }
+}
+
 XSCREENSAVER_MODULE_2 ("GLSlideshow", glslideshow, slideshow)
 
 #endif /* USE_GL */
