@@ -39,8 +39,8 @@
 # define DEFAULTS           "*delay: 20000\n" \
                             "*showFPS: False\n" \
                             "*wireframe: False\n" \
+			    "*suppressRotationAnimation: True\n" \
 
-# define refresh_jigglypuff 0
 # define release_jigglypuff 0
 # include "xlockmore.h"
 #else
@@ -51,9 +51,9 @@
 # include "config.h"
 #endif
 
-#include "xpm-ximage.h"
+#include "ximage-loader.h"
 #include "gltrackball.h"
-#include "../images/jigglymap.xpm"
+#include "images/gen/jigglymap_png.h"
 
 #ifdef USE_GL
 
@@ -93,7 +93,14 @@ static int shininess;
 
 static int random_parms;
 
+/* This is all the half-edge b-rep code (as well as basic geometry) */
 typedef struct solid solid;
+typedef struct face face;
+typedef struct edge edge;
+typedef struct hedge hedge;
+typedef struct vertex vertex;
+typedef GLfloat coord;
+typedef coord vector[3];
 
 typedef struct {
     float stable_distance;
@@ -118,6 +125,13 @@ typedef struct {
     float axis;
     float speed;
 
+    face *faces;
+    edge *edges;
+    hedge *hedges;
+    vertex *vertices;
+
+    GLuint texid;
+
     GLXContext *glx_context;
 } jigglystruct;
 
@@ -202,14 +216,6 @@ static const GLfloat flowerbox_colors[4][4] = {
 #endif
 #endif /* 0 */
 
-/* This is all the half-edge b-rep code (as well as basic geometry) */
-typedef struct face face;
-typedef struct edge edge;
-typedef struct hedge hedge;
-typedef struct vertex vertex;
-typedef GLfloat coord;
-typedef coord vector[3];
-
 struct solid {
     face *faces;
     edge *edges;
@@ -220,21 +226,21 @@ struct face {
     solid *s;
     hedge *start;
     const GLfloat *color;
-    face *next;
+    face *next, *next0;
 };
 
 struct edge {
     solid *s;
     hedge *left;
     hedge *right;
-    edge *next;
+    edge *next, *next0;
 };
 
 struct hedge {
     face *f;
     edge *e;
     vertex *vtx;
-    hedge *next;
+    hedge *next, *next0;
     hedge *prev;
 };
 
@@ -245,7 +251,7 @@ struct vertex {
     vector n;
     vector f;
     vector vel;
-    vertex *next;
+    vertex *next, *next0;
 };
 
 static inline void vector_init(vector v, coord x, coord y, coord z)
@@ -286,10 +292,12 @@ static inline void vector_scale(vector v, coord s)
     v[2] *= s;
 }
 
+/*
 static inline coord dot(vector v1, vector v2)
 {
     return v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2];
 }
+*/
 
 static inline void cross(vector v1, vector v2, vector v)
 {
@@ -350,12 +358,14 @@ static inline hedge *partner(hedge *h) {
     }
 }
 
-static vertex *vertex_new(solid *s, vector v)
+static vertex *vertex_new(jigglystruct *js, solid *s, vector v)
 {
     vertex *vtx = (vertex*)malloc(sizeof(vertex));
 
     if(!vtx)
 	return NULL;
+    vtx->next0 = js->vertices;
+    js->vertices = vtx;
     vtx->s = s;
     vtx->next = s->vertices;
     s->vertices = vtx;
@@ -369,7 +379,7 @@ static vertex *vertex_new(solid *s, vector v)
  * i.e. it is a helper for the split_* functions, which
  * maintain the consistency of the solid.
  */
-static hedge *hedge_new(hedge *hafter, vertex *vtx)
+static hedge *hedge_new(jigglystruct *js, hedge *hafter, vertex *vtx)
 {
     hedge *h = (hedge*)malloc(sizeof(hedge));
     
@@ -377,6 +387,8 @@ static hedge *hedge_new(hedge *hafter, vertex *vtx)
 /*	_DEBUG("Out of memory in hedge_new()\n",NULL); */
 	return NULL;
     }
+    h->next0 = js->hedges;
+    js->hedges = h;
     h->f = hafter->f;
     h->vtx = vtx;
     h->e = NULL;
@@ -387,13 +399,15 @@ static hedge *hedge_new(hedge *hafter, vertex *vtx)
     return h;
 }
 
-static edge *edge_new(solid *s)
+static edge *edge_new(jigglystruct *js, solid *s)
 {
     edge *e = (edge*)malloc(sizeof(edge));
     if(!e) {
 /*	_DEBUG("Out of memory in edge_new()\n",NULL);*/
 	exit(-1);
     }
+    e->next0 = js->edges;
+    js->edges = e;
     e->next = s->edges;
     s->edges = e;
     e-> s = s;
@@ -401,13 +415,15 @@ static edge *edge_new(solid *s)
     return e;
 }
 
-static face *face_new(solid *s, hedge *h)
+static face *face_new(jigglystruct *js, solid *s, hedge *h)
 {
     face *f = (face*)malloc(sizeof(face));
     if(!f) {
 /*	_DEBUG("Out of memory in face_new()",NULL);*/
 	exit(-1);
     }
+    f->next0 = js->faces;
+    js->faces = f;
     f->s = s;
     f->start = h;
     f->next = s->faces;
@@ -429,7 +445,7 @@ static face *face_new(solid *s, hedge *h)
  *    the vertex returned will have h==<the new halfedge>.
  */
 
-static vertex *vertex_split(hedge *h, vector v)
+static vertex *vertex_split(jigglystruct *js, hedge *h, vector v)
 {
     hedge *h2, *hn1, *hn2;
     vertex *vtxn;
@@ -439,10 +455,10 @@ static vertex *vertex_split(hedge *h, vector v)
     f1 = h->f;
     h2 = partner(h);
     
-    vtxn = vertex_new(f1->s, v);
-    hn1 = hedge_new(h, vtxn);
+    vtxn = vertex_new(js, f1->s, v);
+    hn1 = hedge_new(js, h, vtxn);
     vtxn->h = hn1;
-    hn2 = hedge_new(h2, vtxn);
+    hn2 = hedge_new(js, h2, vtxn);
     hn2->e = h->e;
 
     if(h2->e->left == h2)
@@ -450,7 +466,7 @@ static vertex *vertex_split(hedge *h, vector v)
     else
 	h2->e->right = hn2;
 
-    en = edge_new(f1->s);
+    en = edge_new(js, f1->s);
     en->left = hn1;
     en->right = h2;
     hn1->e = en;
@@ -458,7 +474,7 @@ static vertex *vertex_split(hedge *h, vector v)
     return vtxn;
 }
 
-static face *face_split(face *f, hedge *h1, hedge *h2)
+static face *face_split(jigglystruct *js, face *f, hedge *h1, hedge *h2)
 {
     hedge *hn1, *hn2, *tmp;
     edge *en;
@@ -479,9 +495,9 @@ static face *face_split(face *f, hedge *h1, hedge *h2)
     h1->prev = h2->prev;
     h2->prev = tmp;
     /* insert halfedges & create edge */
-    hn1 = hedge_new(h2->prev, h1->vtx);
-    hn2 = hedge_new(h1->prev, h2->vtx);
-    en = edge_new(f->s);
+    hn1 = hedge_new(js, h2->prev, h1->vtx);
+    hn2 = hedge_new(js, h1->prev, h2->vtx);
+    en = edge_new(js, f->s);
     en->left = hn1;
     en->right = hn2;
     hn1->e = en;
@@ -493,7 +509,7 @@ static face *face_split(face *f, hedge *h1, hedge *h2)
     while(tmp != h1 && tmp != h2)
 	tmp = tmp->next;
     tmp = (tmp == h1) ? h2 : h1 ;
-    fn = face_new(f->s, tmp);
+    fn = face_new(js, f->s, tmp);
     do {
 	tmp->f = fn;
 	tmp = tmp->next;
@@ -502,7 +518,7 @@ static face *face_split(face *f, hedge *h1, hedge *h2)
     return fn;
 }
 
-static solid *solid_new(vector where) 
+static solid *solid_new(jigglystruct *js, vector where) 
 {
     solid *s = (solid*)malloc(sizeof(solid));
     face *f1, *f2;
@@ -519,19 +535,24 @@ static solid *solid_new(vector where)
     h1->next = h1->prev = h1;
     h2->next = h2->prev = h2;
 
-    vtx = vertex_new(s, where);
+    h1->next0 = js->hedges;
+    js->hedges = h1;
+    h2->next0 = js->hedges;
+    js->hedges = h2;
+
+    vtx = vertex_new(js, s, where);
     vtx->h = h1;
     h1->vtx = vtx;
     h2->vtx = vtx;
 
-    e = edge_new(s);
+    e = edge_new(js, s);
     e->left = h1;
     e->right = h2;
     h1->e = e;
     h2->e = e;
 
-    f1 = face_new(s, h1);
-    f2 = face_new(s, h2);
+    f1 = face_new(js, s, h1);
+    f2 = face_new(js, s, h2);
     h1->f = f1;
     h2->f = f2;
 
@@ -539,14 +560,14 @@ static solid *solid_new(vector where)
 }
 
 /* This is all the code directly related to constructing the jigglypuff */
-static void face_tessel2(face *f)
+static void face_tessel2(jigglystruct *js, face *f)
 {
     hedge *h1=f->start->prev, *h2=f->start->next;
     
     if(h1->next == h1)
 	return;
     while(h2 != h1 && h2->next != h1) {
-	f = face_split(f, h1, h2);
+	f = face_split(js, f, h1, h2);
 	h1 = f->start;
 	h2 = f->start->next->next;
     }
@@ -560,7 +581,7 @@ static void face_tessel2(face *f)
  * added at the head of the list. If that ever changes,
  * this is borked. 
  */
-static void solid_tesselate(solid *s) 
+static void solid_tesselate(jigglystruct *js, solid *s) 
 {
     edge *e = s->edges;
     face *f = s->faces;
@@ -568,11 +589,11 @@ static void solid_tesselate(solid *s)
     while(e) {
 	vector v;
 	midpoint(e->left->vtx->v, e->right->vtx->v, v);
-	vertex_split(e->left, v);
+	vertex_split(js, e->left, v);
 	e = e->next;
     }
     while(f) {
-	face_tessel2(f);
+	face_tessel2(js, f);
 	f=f->next;
     }
 }
@@ -597,19 +618,19 @@ static solid *tetrahedron(jigglystruct *js)
     int i;
 
     vector_init(v, 1, 1, 1);
-    s = solid_new(v);
+    s = solid_new(js, v);
     vector_init(v, -1, -1, 1);
     h = s->faces->start;
-    vtx = vertex_split(h, v);
+    vtx = vertex_split(js, h, v);
     vector_init(v, -1, 1, -1);
-    vtx = vertex_split(vtx->h, v);
+    vtx = vertex_split(js, vtx->h, v);
     h = vtx->h;
-    f = face_split(s->faces, h, h->prev);
+    f = face_split(js, s->faces, h, h->prev);
     vector_init(v, 1, -1, -1);
-    vertex_split(f->start, v);
+    vertex_split(js, f->start, v);
     f = s->faces->next->next;
     h = f->start;
-    face_split(f, h, h->next->next);
+    face_split(js, f, h, h->next->next);
 
     if(js->color_style == COLOR_STYLE_FLOWERBOX) {
 	f = s->faces;
@@ -627,7 +648,7 @@ static solid *tesselated_tetrahedron(coord size, int iter, jigglystruct *js) {
     int i;
 
     for(i=0; i<iter; i++) {
-	solid_tesselate(s);
+	solid_tesselate(js, s);
     }
     return s;
 }
@@ -765,9 +786,12 @@ static void update_shape(jigglystruct *js)
 
 static void init_texture(ModeInfo *mi)
 {
-    XImage *img = xpm_to_ximage(mi->dpy, mi->xgwa.visual,
-			       mi->xgwa.colormap, jigglymap_xpm);
+    jigglystruct *js = &jss[MI_SCREEN(mi)];
+    XImage *img = image_data_to_ximage(mi->dpy, mi->xgwa.visual,
+                                       jigglymap_png, sizeof(jigglymap_png));
 
+    glGenTextures (1, &js->texid);
+    glBindTexture (GL_TEXTURE_2D, js->texid);
     glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB,
 		 img->width, img->height, 0, GL_RGBA,
 		 GL_UNSIGNED_BYTE, img->data);
@@ -784,7 +808,6 @@ static void setup_opengl(ModeInfo *mi, jigglystruct *js)
     const GLfloat scolor[4]= {0.9f, 0.9f, 0.9f, 0.5f};
 
     glDrawBuffer(GL_BACK);
-    glClearColor(0, 0, 0, 0);
     glShadeModel(GL_SMOOTH);
     glEnable(GL_DEPTH_TEST);
 
@@ -839,10 +862,12 @@ static int parse_color(jigglystruct *js)
 	js->color_style = COLOR_STYLE_FLOWERBOX;
 	return 1;
     }
+# ifndef HAVE_JWZGLES  /* SPHERE_MAP unimplemented */
     else if(!strcmp(color, "chrome")) {
 	js->color_style = COLOR_STYLE_CHROME;
 	return 1;
     }
+# endif
     else if(!strcmp(color, "cycle")) {
 	js->color_style = COLOR_STYLE_CYCLE;
 	js->jiggly_color[0] = ((float)random()) / REAL_RAND_MAX * 0.7 + 0.3;
@@ -871,8 +896,14 @@ static int parse_color(jigglystruct *js)
 
 static void randomize_parameters(jigglystruct *js) {
     do_tetrahedron = random() & 1;
+# ifndef HAVE_JWZGLES /* #### glPolygonMode other than GL_FILL unimplemented */
     js->do_wireframe = !(random() & 3);
+# endif
     js->color_style = random() % 5;
+# ifdef HAVE_JWZGLES  /* #### SPHERE_MAP unimplemented */
+    while (js->color_style == COLOR_STYLE_CHROME)
+      js->color_style = random() % 5;;
+# endif
     if(js->color_style == COLOR_STYLE_NORMAL
 	|| js->color_style == COLOR_STYLE_CYCLE) {
 	js->jiggly_color[0] = ((float)random()) / REAL_RAND_MAX * 0.5 + 0.5;
@@ -920,52 +951,36 @@ ENTRYPOINT Bool jigglypuff_handle_event(ModeInfo *mi, XEvent *event)
 {
     jigglystruct *js = &jss[MI_SCREEN(mi)];
     
-    if(event->xany.type == ButtonPress &&
-       event->xbutton.button == Button1) {
-	js->button_down = 1;
-	gltrackball_start(js->trackball, event->xbutton.x, event->xbutton.y,
-			  MI_WIDTH(mi), MI_HEIGHT(mi));
-	return True;
-    }
-    else if(event->xany.type == ButtonRelease &&
-	    event->xbutton.button == Button1) {
-	js->button_down = 0;
-	return True;
-    }
-  else if (event->xany.type == ButtonPress &&
-           (event->xbutton.button == Button4 ||
-            event->xbutton.button == Button5 ||
-            event->xbutton.button == Button6 ||
-            event->xbutton.button == Button7))
-    {
-      gltrackball_mousewheel (js->trackball, event->xbutton.button, 10,
-                              !!event->xbutton.state);
-      return True;
-    }
-    else if(event->xany.type == MotionNotify && js->button_down) {
-	gltrackball_track(js->trackball, event->xmotion.x, event->xmotion.y,
-			  MI_WIDTH(mi), MI_HEIGHT(mi));
-	return True;
-    }
+    if (gltrackball_event_handler (event, js->trackball,
+                                   MI_WIDTH (mi), MI_HEIGHT (mi),
+                                   &js->button_down))
+    return True;
+
     return False;
 }
 
 ENTRYPOINT void reshape_jigglypuff(ModeInfo *mi, int width, int height)
 {
-    GLfloat aspect = (GLfloat)width / (GLfloat)height;
+  double h = (GLfloat) height / (GLfloat) width;  
+  int y = 0;
 
-    glViewport(0, 0, width, height);
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    glFrustum(-0.5*aspect, 0.5*aspect, -0.5, 0.5, 1, 20);
-/*    glTranslatef(0, 0, -10);*/
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport(0, y, width, height);
+  glMatrixMode(GL_PROJECTION);
+  glLoadIdentity();
+  glFrustum(-0.5*(1/h), 0.5*(1/h), -0.5, 0.5, 1, 20);
 }
 
 ENTRYPOINT void draw_jigglypuff(ModeInfo *mi)
 {
     jigglystruct *js = &jss[MI_SCREEN(mi)];
     
-    glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(js->glx_context));
+    glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *js->glx_context);
     
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -973,6 +988,16 @@ ENTRYPOINT void draw_jigglypuff(ModeInfo *mi)
     glLoadIdentity();
     glTranslatef(0,0,-10);
 
+
+# ifdef HAVE_MOBILE	/* Keep it the same relative size when rotated. */
+    {
+      GLfloat h = MI_HEIGHT(mi) / (GLfloat) MI_WIDTH(mi);
+      int o = (int) current_device_rotation();
+      if (o != 0 && o != 180 && o != -180)
+        glScalef (1/h, 1/h, 1/h);
+    }
+# endif
+
     glRotatef(js->angle, sin(js->axis), cos(js->axis), -sin(js->axis));
     glTranslatef(0, 0, 5);
     if(!(js->button_down)) {
@@ -983,6 +1008,7 @@ ENTRYPOINT void draw_jigglypuff(ModeInfo *mi)
 	    js->axis -= 2*M_PI;
 	}
     }
+
     gltrackball_rotate(js->trackball);
 
     if(js->color_style == COLOR_STYLE_CYCLE) {
@@ -1011,18 +1037,14 @@ ENTRYPOINT void init_jigglypuff(ModeInfo *mi)
     jigglystruct *js;
     int subdivs;
 
-    if(!jss) {
-	jss = (jigglystruct*)
-	    calloc(MI_NUM_SCREENS(mi), sizeof(jigglystruct));
-	if(!jss) {
-	    fprintf(stderr, "%s: No..memory...must...abort..\n", progname);
-	    exit(1);
-	}
-    }
+    MI_INIT(mi, jss);
 
     js = &jss[MI_SCREEN(mi)];
 
     js->do_wireframe = MI_IS_WIREFRAME(mi);
+# ifdef HAVE_JWZGLES
+    js->do_wireframe = 0; /* GL_LINE unimplemented */
+# endif
 
     js->shininess = shininess;
 
@@ -1053,14 +1075,14 @@ ENTRYPOINT void init_jigglypuff(ModeInfo *mi)
     calculate_parameters(js, subdivs);
 
     if((js->glx_context = init_GL(mi)) != NULL) {
-	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(js->glx_context));
+	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *js->glx_context);
 	setup_opengl(mi, js);
 	reshape_jigglypuff(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
     }
     else {
 	MI_CLEARWINDOW(mi);
     }
-    js->trackball = gltrackball_init();
+    js->trackball = gltrackball_init(True);
 /*    _DEBUG("distance : %f\nhold : %f\nspherify : %f\ndamping : %f\ndfact : %f\n",
 	   js->stable_distance, js->hold_strength, js->spherify_strength,
 	   js->damping_velocity, js->damping_factor);
@@ -1068,6 +1090,41 @@ ENTRYPOINT void init_jigglypuff(ModeInfo *mi)
 	   js->do_wireframe, js->spooky, js->color_style, js->shininess);*/
 }
 
+
+ENTRYPOINT void free_jigglypuff(ModeInfo *mi)
+{
+    jigglystruct *js = &jss[MI_SCREEN(mi)];
+
+    if (!js->glx_context) return;
+    glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *js->glx_context);
+
+    if (js->texid) glDeleteTextures(1, &js->texid);
+    if (js->trackball) gltrackball_free (js->trackball);
+
+    while (js->faces) {
+      face *n = js->faces->next0;
+      free (js->faces);
+      js->faces = n;
+    }
+    while (js->edges) {
+      edge *n = js->edges->next0;
+      free (js->edges);
+      js->edges = n;
+    }
+    while (js->hedges) {
+      hedge *n = js->hedges->next0;
+      free (js->hedges);
+      js->hedges = n;
+    }
+    while (js->vertices) {
+      vertex *n = js->vertices->next0;
+      free (js->vertices);
+      js->vertices = n;
+    }
+    free (js->shape);
+    if (js->texid) glDeleteTextures (1, &js->texid);
+}
+
 XSCREENSAVER_MODULE ("JigglyPuff", jigglypuff)
 
 #endif /* USE_GL */
