@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1997-2012 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1997-2019 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -48,22 +48,22 @@
 #define DEF_WOBBLE       "True"
 #define DEF_DEBUG        "False"
 
-#define DEF_FONT "-*-helvetica-bold-r-normal-*-240-*"
+#define DEF_FONT "-*-helvetica-bold-r-normal-*-*-240-*-*-*-*-*-*"
 #define DEFAULTS  "*delay:		20000	\n" \
 		  "*showFPS:		False	\n" \
 		  "*font:	     " DEF_FONT"\n" \
 		  "*wireframe:		False	\n" \
 		  "*desktopGrabber:   xscreensaver-getimage -no-desktop %s\n" \
 		  "*grabDesktopImages:	False	\n" \
-		  "*chooseRandomImages:	True	\n"
+		  "*chooseRandomImages:	True	\n" \
+		  "*suppressRotationAnimation: True\n" \
 
 
-# define refresh_jigsaw 0
 # define release_jigsaw 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
 
-#ifdef HAVE_COCOA
+#ifdef HAVE_JWXYZ
 # include "jwxyz.h"
 #else
 # include <X11/Xlib.h>
@@ -555,7 +555,7 @@ draw_piece (jigsaw_configuration *jc, puzzle_piece *p,
 # else  /* !HAVE_TESS */
 
       GLfloat *tri = (GLfloat *)
-        (GLfloat *) malloc (s->n_points * 4 * 3 * 3 * sizeof(*pts));
+        malloc (s->n_points * 4 * 3 * 3 * sizeof(*tri));
       GLfloat *otri = tri;
       int count;
       GLdouble zz;
@@ -823,8 +823,8 @@ image_loaded_cb (const char *filename, XRectangle *geometry,
 
   glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
   glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
-  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 
   make_puzzle_grid (mi);
 }
@@ -1157,8 +1157,11 @@ loading_msg (ModeInfo *mi)
   jigsaw_configuration *jc = &sps[MI_SCREEN(mi)];
   int wire = MI_IS_WIREFRAME(mi);
   const char *text = "Loading...";
-  int h;
-  int w = texture_string_width (jc->texfont, text, &h);
+  XCharStruct e;
+  int w, h;
+  texture_string_metrics (jc->texfont, text, &e, 0, 0);
+  w = e.width;
+  h = e.ascent + e.descent;
 
   if (wire) return;
 
@@ -1321,8 +1324,15 @@ reshape_jigsaw (ModeInfo *mi, int width, int height)
 {
   jigsaw_configuration *jc = &sps[MI_SCREEN(mi)];
   GLfloat h = (GLfloat) height / (GLfloat) width;
+  int y = 0;
 
-  glViewport (0, 0, (GLint) width, (GLint) height);
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport (0, y, (GLint) width, (GLint) height);
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -1334,6 +1344,14 @@ reshape_jigsaw (ModeInfo *mi, int width, int height)
              0.0, 0.0, 0.0,
              0.0, 1.0, 0.0);
 
+# ifdef HAVE_MOBILE	/* Keep it the same relative size when rotated. */
+  {
+    int o = (int) current_device_rotation();
+    if (o != 0 && o != 180 && o != -180)
+      glScalef (1/h, 1/h, 1/h);
+  }
+# endif
+
   glClear(GL_COLOR_BUFFER_BIT);
 
   jc->line_thickness = (MI_IS_WIREFRAME (mi) ? 1 : MAX (1, height / 300.0));
@@ -1345,53 +1363,16 @@ jigsaw_handle_event (ModeInfo *mi, XEvent *event)
 {
   jigsaw_configuration *jc = &sps[MI_SCREEN(mi)];
 
-  if (event->xany.type == ButtonPress &&
-      event->xbutton.button == Button1)
+  if (gltrackball_event_handler (event, jc->trackball,
+                                 MI_WIDTH (mi), MI_HEIGHT (mi),
+                                 &jc->button_down_p))
+    return True;
+  else if (screenhack_event_helper (MI_DISPLAY(mi), MI_WINDOW(mi), event))
     {
-      jc->button_down_p = True;
-      gltrackball_start (jc->trackball,
-                         event->xbutton.x, event->xbutton.y,
-                         MI_WIDTH (mi), MI_HEIGHT (mi));
+      begin_scatter (mi, False);
+      jc->state = PUZZLE_SCATTER;
       return True;
     }
-  else if (event->xany.type == ButtonRelease &&
-           event->xbutton.button == Button1)
-    {
-      jc->button_down_p = False;
-      return True;
-    }
-  else if (event->xany.type == ButtonPress &&
-           (event->xbutton.button == Button4 ||
-            event->xbutton.button == Button5 ||
-            event->xbutton.button == Button6 ||
-            event->xbutton.button == Button7))
-    {
-      gltrackball_mousewheel (jc->trackball, event->xbutton.button, 10,
-                              !!event->xbutton.state);
-      return True;
-    }
-  else if (event->xany.type == MotionNotify &&
-           jc->button_down_p)
-    {
-      gltrackball_track (jc->trackball,
-                         event->xmotion.x, event->xmotion.y,
-                         MI_WIDTH (mi), MI_HEIGHT (mi));
-      return True;
-    }
-  else if (event->xany.type == KeyPress)
-    {
-      KeySym keysym;
-      char c = 0;
-      XLookupString (&event->xkey, &c, 1, &keysym, 0);
-
-      if (c == ' ' || c == '\t' || c == '\r' || c == '\n')
-        {
-          begin_scatter (mi, False);
-          jc->state = PUZZLE_SCATTER;
-          return True;
-        }
-    }
-
 
   return False;
 }
@@ -1403,14 +1384,7 @@ init_jigsaw (ModeInfo *mi)
   jigsaw_configuration *jc;
   int wire = MI_IS_WIREFRAME(mi);
 
-  if (!sps) {
-    sps = (jigsaw_configuration *)
-      calloc (MI_NUM_SCREENS(mi), sizeof (jigsaw_configuration));
-    if (!sps) {
-      fprintf(stderr, "%s: out of memory\n", progname);
-      exit(1);
-    }
-  }
+  MI_INIT (mi, sps);
   jc = &sps[MI_SCREEN(mi)];
   jc->glx_context = init_GL(mi);
 
@@ -1429,7 +1403,7 @@ init_jigsaw (ModeInfo *mi)
       glLightfv(GL_LIGHT0, GL_SPECULAR, spc);
     }
 
-  jc->trackball = gltrackball_init ();
+  jc->trackball = gltrackball_init (False);
   jc->rot = make_rotator (0, 0, 0, 0, speed * 0.002, True);
   jc->texfont = load_texture_font (MI_DISPLAY(mi), "font");
 
@@ -1461,14 +1435,14 @@ draw_jigsaw (ModeInfo *mi)
   if (!jc->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(jc->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *jc->glx_context);
 
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
   mi->polygon_count = 0;
 
   glPushMatrix ();
-  glRotatef(current_device_rotation(), 0, 0, 1);
+/*  glRotatef(current_device_rotation(), 0, 0, 1); */
   gltrackball_rotate (jc->trackball);
 
   animate (mi);
@@ -1529,6 +1503,22 @@ draw_jigsaw (ModeInfo *mi)
   glXSwapBuffers(dpy, window);
 }
 
+
+ENTRYPOINT void
+free_jigsaw (ModeInfo *mi)
+{
+  jigsaw_configuration *jc = &sps[MI_SCREEN(mi)];
+
+  if (!jc->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *jc->glx_context);
+
+  if (jc->trackball) free (jc->trackball);
+  if (jc->rot) free_rotator (jc->rot);
+  if (jc->texfont) free_texture_font (jc->texfont);
+  free_puzzle_grid (jc);
+  if (glIsList(jc->loading_dlist)) glDeleteLists(jc->loading_dlist, 1);
+}
+
 XSCREENSAVER_MODULE ("Jigsaw", jigsaw)
 
 #endif /* USE_GL */
