@@ -21,10 +21,14 @@
 #include <stdio.h>
 #include <math.h>
 
-#ifndef HAVE_COCOA
+#ifndef HAVE_JWXYZ
 # include <GL/gl.h>
 #endif
 
+#ifdef HAVE_ANDROID
+# include <GLES/gl.h>
+#endif
+
 #ifdef HAVE_JWZGLES
 # include "jwzgles.h"
 #endif /* HAVE_JWZGLES */
