@@ -1,4 +1,4 @@
-/* molecule, Copyright (c) 2001-2012 Jamie Zawinski <jwz@jwz.org>
+/* molecule, Copyright (c) 2001-2016 Jamie Zawinski <jwz@jwz.org>
  * Draws molecules, based on coordinates from PDB (Protein Data Base) files.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -12,7 +12,7 @@
 
 
 /* Documentation on the PDB file format:
-   http://en.wikipedia.org/wiki/Protein_Data_Bank_%28file_format%29
+   https://en.wikipedia.org/wiki/Protein_Data_Bank_%28file_format%29
    http://www.wwpdb.org/docs.html
    http://www.wwpdb.org/documentation/format32/v3.2.html
    http://www.wwpdb.org/documentation/format32/sect9.html
@@ -24,22 +24,15 @@
    http://www.wwpdb.org/docs.html
  */
 
-#ifdef HAVE_GLBITMAP
-# define ATOM_FONT "-*-helvetica-medium-r-normal-*-180-*"
-#else
-# define ATOM_FONT "-*-helvetica-medium-r-normal-*-240-*"
-#endif
-
 #define DEFAULTS	"*delay:	10000         \n" \
 			"*showFPS:      False         \n" \
 			"*wireframe:    False         \n" \
-			"*atomFont:   " ATOM_FONT "\n" \
-			"*atomFont2:  -*-helvetica-bold-r-normal-*-80-*\n" \
-			"*titleFont:  -*-helvetica-medium-r-normal-*-180-*\n" \
-			"*noLabelThreshold:    30     \n" \
+	"*atomFont:   -*-helvetica-medium-r-normal-*-*-240-*-*-*-*-*-*\n" \
+	"*titleFont:  -*-helvetica-medium-r-normal-*-*-180-*-*-*-*-*-*\n" \
+			"*noLabelThreshold:    150    \n" \
 			"*wireframeThreshold:  150    \n" \
+			"*suppressRotationAnimation: True\n" \
 
-# define refresh_molecule 0
 # define release_molecule 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
@@ -48,7 +41,7 @@
 #include "colors.h"
 #include "sphere.h"
 #include "tube.h"
-#include "glxfonts.h"
+#include "texfont.h"
 #include "rotator.h"
 #include "gltrackball.h"
 
@@ -98,7 +91,7 @@ static const char * const builtin_pdb_data[] = {
 };
 
 
-#ifndef USE_IPHONE
+#ifndef HAVE_MOBILE
 # define LOAD_FILES
 #endif
 
@@ -130,7 +123,7 @@ static const atom_data all_atom_data[] = {
 
 typedef struct {
   int id;		/* sequence number in the PDB file */
-  const char *label;	/* The atom name */
+  char *label;		/* The atom name */
   GLfloat x, y, z;	/* position in 3-space (angstroms) */
   const atom_data *data; /* computed: which style of atom this is */
 } molecule_atom;
@@ -142,7 +135,7 @@ typedef struct {
 
 
 typedef struct {
-  const char *label;		/* description of this compound */
+  char *label;		/* description of this compound */
   int natoms, atoms_size;
   int nbonds, bonds_size;
   molecule_atom *atoms;
@@ -167,24 +160,20 @@ typedef struct {
 
   int mode;  /* 0 = normal, 1 = out, 2 = in */
   int mode_tick;
+  int next;  /* 0 = random, -1 = back, 1 = forward */
 
   GLuint molecule_dlist;
   GLuint shell_dlist;
 
-# ifdef HAVE_GLBITMAP
-  XFontStruct *xfont1, *xfont2, *xfont3;
-  GLuint font1_dlist, font2_dlist, font3_dlist;
-# else
-  texture_font_data *font1_data, *font2_data, *font3_data;
-# endif
-
+  texture_font_data *atom_font, *title_font;
 
   int polygon_count;
 
   time_t draw_time;
   int draw_tick;
 
-  int scale_down;
+  GLfloat overall_scale;
+  int low_rez_p;
 
 } molecule_configuration;
 
@@ -258,8 +247,8 @@ static int
 sphere (molecule_configuration *mc,
         GLfloat x, GLfloat y, GLfloat z, GLfloat diameter, Bool wire)
 {
-  int stacks = (mc->scale_down ? SPHERE_STACKS_2 : SPHERE_STACKS);
-  int slices = (mc->scale_down ? SPHERE_SLICES_2 : SPHERE_SLICES);
+  int stacks = (mc->low_rez_p ? SPHERE_STACKS_2 : SPHERE_STACKS);
+  int slices = (mc->low_rez_p ? SPHERE_SLICES_2 : SPHERE_SLICES);
 
   glPushMatrix ();
   glTranslatef (x, y, z);
@@ -275,15 +264,8 @@ static void
 load_fonts (ModeInfo *mi)
 {
   molecule_configuration *mc = &mcs[MI_SCREEN(mi)];
-# ifdef HAVE_GLBITMAP
-  load_font (mi->dpy, "atomFont",  &mc->xfont1, &mc->font1_dlist);
-  load_font (mi->dpy, "atomFont2", &mc->xfont2, &mc->font2_dlist);
-  load_font (mi->dpy, "titleFont", &mc->xfont3, &mc->font3_dlist);
-# else
-  mc->font1_data = load_texture_font (mi->dpy, "atomFont");
-  mc->font2_data = load_texture_font (mi->dpy, "atomFont2");
-  mc->font3_data = load_texture_font (mi->dpy, "titleFont");
-# endif
+  mc->atom_font = load_texture_font (mi->dpy, "atomFont");
+  mc->title_font = load_texture_font (mi->dpy, "titleFont");
 }
 
 
@@ -497,9 +479,12 @@ draw_bounding_box (ModeInfo *mi)
 
   glColor3f (c2[0], c2[1], c2[2]);
   glBegin(GL_LINES);
-  if (x1 > 0) x1 = 0; if (x2 < 0) x2 = 0;
-  if (y1 > 0) y1 = 0; if (y2 < 0) y2 = 0;
-  if (z1 > 0) z1 = 0; if (z2 < 0) z2 = 0;
+  if (x1 > 0) x1 = 0;
+  if (x2 < 0) x2 = 0;
+  if (y1 > 0) y1 = 0;
+  if (y2 < 0) y2 = 0;
+  if (z1 > 0) z1 = 0;
+  if (z2 < 0) z2 = 0;
   glVertex3f(x1, 0,  0);  glVertex3f(x2, 0,  0); 
   glVertex3f(0 , y1, 0);  glVertex3f(0,  y2, 0); 
   glVertex3f(0,  0,  z1); glVertex3f(0,  0,  z2); 
@@ -535,14 +520,15 @@ ensure_bounding_box_visible (ModeInfo *mi)
 
   mc->molecule_size = size;
 
-  mc->scale_down = 0;
+  mc->low_rez_p = 0;
+  mc->overall_scale = 1;
 
   if (size > max_size)
     {
-      GLfloat scale = max_size / size;
-      glScalef (scale, scale, scale);
+      mc->overall_scale = max_size / size;
+      glScalef (mc->overall_scale, mc->overall_scale, mc->overall_scale);
 
-      mc->scale_down = scale < 0.3;
+      mc->low_rez_p = mc->overall_scale < 0.3;
     }
 
   glTranslatef (-(x1 + w/2),
@@ -604,21 +590,23 @@ build_molecule (ModeInfo *mi, Bool transparent_p)
           }
         else
           {
-            int faces = (mc->scale_down ? TUBE_FACES_2 : TUBE_FACES);
+            int faces = (mc->low_rez_p ? TUBE_FACES_2 : TUBE_FACES);
 # ifdef SMOOTH_TUBE
             int smooth = True;
 # else
             int smooth = False;
 # endif
-            GLfloat thickness = 0.07 * b->strength;
-            GLfloat cap_size = 0.03;
+            Bool cap_p = (!do_atoms || do_shells);
+            GLfloat base = 0.07;
+            GLfloat thickness = base * b->strength;
+            GLfloat cap_size = (cap_p ? base / 2 : 0);
             if (thickness > 0.3)
               thickness = 0.3;
 
             polys += tube (from->x, from->y, from->z,
                            to->x,   to->y,   to->z,
                            thickness, cap_size,
-                           faces, smooth, (!do_atoms || do_shells), wire);
+                           faces, smooth, cap_p, wire);
           }
       }
 
@@ -646,7 +634,7 @@ build_molecule (ModeInfo *mi, Bool transparent_p)
 
 static void
 push_atom (molecule *m,
-           int id, const char *label,
+           int id, char *label,
            GLfloat x, GLfloat y, GLfloat z)
 {
   m->natoms++;
@@ -714,10 +702,7 @@ parse_pdb_data (molecule *m, const char *data, const char *filename, int line)
         {
           char *name = calloc (1, 100);
           char *n2 = name;
-          int L = strlen(s);
-          if (L > 99) L = 99;
-
-          strncpy (n2, s, L);
+          sprintf (n2, "%.99s", s);
           n2 += 7;
           while (isspace(*n2)) n2++;
 
@@ -734,7 +719,7 @@ parse_pdb_data (molecule *m, const char *data, const char *filename, int line)
               !strcmp (n2 + strlen(n2) - 4, ".pdb"))
             n2[strlen(n2)-4] = 0;
 
-          if (m->label) free ((char *) m->label);
+          if (m->label) free (m->label);
           m->label = strdup (n2);
           free (name);
         }
@@ -791,7 +776,8 @@ parse_pdb_data (molecule *m, const char *data, const char *filename, int line)
           int id;
           const char *end = strchr (s, '\n');
           int L = end - s;
-          char *name = (char *) calloc (1, 4);
+          char *oname = (char *) calloc (1, 4);
+          char *name = oname;
           GLfloat x = -999, y = -999, z = -999;
 
           if (1 != sscanf (s+7, " %d ", &id))
@@ -827,12 +813,14 @@ parse_pdb_data (molecule *m, const char *data, const char *filename, int line)
                    progname, filename, line,
                    id, name, x, y, z);
 */
-          push_atom (m, id, name, x, y, z);
+          push_atom (m, id, strdup(name), x, y, z);
+          free (oname);
         }
       else if (!strncmp (s, "HETATM ", 7))
         {
           int id;
-          char *name = (char *) calloc (1, 4);
+          char *oname = (char *) calloc (1, 4);
+          char *name = oname;
           GLfloat x = -999, y = -999, z = -999;
 
           if (1 != sscanf (s+7, " %d ", &id))
@@ -850,7 +838,8 @@ parse_pdb_data (molecule *m, const char *data, const char *filename, int line)
                    progname, filename, line,
                    id, name, x, y, z);
 */
-          push_atom (m, id, name, x, y, z);
+          push_atom (m, id, strdup(name), x, y, z);
+          free (oname);
         }
       else if (!strncmp (s, "CONECT ", 7))
         {
@@ -904,6 +893,7 @@ parse_pdb_file (molecule *m, const char *name)
       char *buf = (char *) malloc(1024 + strlen(name));
       sprintf(buf, "%s: error reading \"%s\"", progname, name);
       perror(buf);
+      free(buf);
       return -1;
     }
 
@@ -1012,7 +1002,7 @@ generate_molecule_formula (molecule *m)
   strcpy (s, m->label);
   strcat (s, "\n");
   strcat (s, buf);
-  free ((char *) m->label);
+  free (m->label);
   free (buf);
   m->label = s;
 }
@@ -1173,6 +1163,7 @@ load_molecules (ModeInfo *mi)
   if (mc->nmolecules == 0)	/* do the builtins if no files */
     {
       mc->nmolecules = countof(builtin_pdb_data);
+      if (mc->molecules) free (mc->molecules);
       mc->molecules = (molecule *) calloc (sizeof (molecule), mc->nmolecules);
       for (i = 0; i < mc->nmolecules; i++)
         {
@@ -1198,8 +1189,15 @@ ENTRYPOINT void
 reshape_molecule (ModeInfo *mi, int width, int height)
 {
   GLfloat h = (GLfloat) height / (GLfloat) width;
+  int y = 0;
 
-  glViewport (0, 0, (GLint) width, (GLint) height);
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport (0, y, (GLint) width, (GLint) height);
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -1211,6 +1209,14 @@ reshape_molecule (ModeInfo *mi, int width, int height)
              0.0, 0.0, 0.0,
              0.0, 1.0, 0.0);
 
+# ifdef HAVE_MOBILE	/* Keep it the same relative size when rotated. */
+  {
+    int o = (int) current_device_rotation();
+    if (o != 0 && o != 180 && o != -180)
+      glScalef (1/h, 1/h, 1/h);
+  }
+# endif
+
   glClear(GL_COLOR_BUFFER_BIT);
 }
 
@@ -1234,15 +1240,11 @@ startup_blurb (ModeInfo *mi)
 {
   molecule_configuration *mc = &mcs[MI_SCREEN(mi)];
   const char *s = "Constructing molecules...";
-  print_gl_string (mi->dpy,
-# ifdef HAVE_GLBITMAP
-                   mc->xfont3, mc->font3_dlist,
-# else
-                   mc->font3_data,
-# endif
-                   mi->xgwa.width, mi->xgwa.height,
-                   10, mi->xgwa.height - 10,
-                   s, False);
+#ifndef HAVE_ANDROID   /* Doesn't work -- causes whole scene to be black */
+  print_texture_label (mi->dpy, mc->title_font,
+                       mi->xgwa.width, mi->xgwa.height,
+                       0, s);
+#endif
   glFinish();
   glXSwapBuffers(MI_DISPLAY(mi), MI_WINDOW(mi));
 }
@@ -1252,53 +1254,40 @@ molecule_handle_event (ModeInfo *mi, XEvent *event)
 {
   molecule_configuration *mc = &mcs[MI_SCREEN(mi)];
 
-  if (event->xany.type == ButtonPress &&
-      event->xbutton.button == Button1)
+  if (gltrackball_event_handler (event, mc->trackball,
+                                 MI_WIDTH (mi), MI_HEIGHT (mi),
+                                 &mc->button_down_p))
+    return True;
+  else
     {
-      mc->button_down_p = True;
-      gltrackball_start (mc->trackball,
-                         event->xbutton.x, event->xbutton.y,
-                         MI_WIDTH (mi), MI_HEIGHT (mi));
-      return True;
-    }
-  else if (event->xany.type == ButtonRelease &&
-           event->xbutton.button == Button1)
-    {
-      mc->button_down_p = False;
-      return True;
-    }
-  else if (event->xany.type == ButtonPress &&
-           (event->xbutton.button == Button4 ||
-            event->xbutton.button == Button5 ||
-            event->xbutton.button == Button6 ||
-            event->xbutton.button == Button7))
-    {
-      gltrackball_mousewheel (mc->trackball, event->xbutton.button, 10,
-                              !!event->xbutton.state);
-      return True;
-    }
-  else if (event->xany.type == KeyPress)
-    {
-      KeySym keysym;
-      char c = 0;
-      XLookupString (&event->xkey, &c, 1, &keysym, 0);
-
-      if (c == ' ' || c == '\t' || c == '\r' || c == '\n')
+      if (event->xany.type == KeyPress)
         {
-          GLfloat speed = 4.0;
+          KeySym keysym;
+          char c = 0;
+          XLookupString (&event->xkey, &c, 1, &keysym, 0);
+          if (c == '<' || c == ',' || c == '-' || c == '_' ||
+              keysym == XK_Left || keysym == XK_Up || keysym == XK_Prior)
+            {
+              mc->next = -1;
+              goto SWITCH;
+            }
+          else if (c == '>' || c == '.' || c == '=' || c == '+' ||
+                   keysym == XK_Right || keysym == XK_Down ||
+                   keysym == XK_Next)
+            {
+              mc->next = 1;
+              goto SWITCH;
+            }
+        }
+
+      if (screenhack_event_helper (MI_DISPLAY(mi), MI_WINDOW(mi), event))
+        {
+        SWITCH:
           mc->mode = 1;
-          mc->mode_tick = 10 * speed;
+          mc->mode_tick = 4;
           return True;
         }
     }
-  else if (event->xany.type == MotionNotify &&
-           mc->button_down_p)
-    {
-      gltrackball_track (mc->trackball,
-                         event->xmotion.x, event->xmotion.y,
-                         MI_WIDTH (mi), MI_HEIGHT (mi));
-      return True;
-    }
 
   return False;
 }
@@ -1310,14 +1299,7 @@ init_molecule (ModeInfo *mi)
   molecule_configuration *mc;
   int wire;
 
-  if (!mcs) {
-    mcs = (molecule_configuration *)
-      calloc (MI_NUM_SCREENS(mi), sizeof (molecule_configuration));
-    if (!mcs) {
-      fprintf(stderr, "%s: out of memory\n", progname);
-      exit(1);
-    }
-  }
+  MI_INIT (mi, mcs);
 
   mc = &mcs[MI_SCREEN(mi)];
 
@@ -1360,9 +1342,13 @@ init_molecule (ModeInfo *mi)
                             spin_accel,
                             do_wander ? wander_speed : 0,
                             (spinx && spiny && spinz));
-    mc->trackball = gltrackball_init ();
+    mc->trackball = gltrackball_init (True);
   }
 
+#ifdef HAVE_ANDROID   /* Doesn't work -- not transparent */
+  do_shells = False;
+#endif
+
   orig_do_labels = do_labels;
   orig_do_atoms  = do_atoms;
   orig_do_bonds  = do_bonds;
@@ -1397,20 +1383,11 @@ draw_labels (ModeInfo *mi)
   molecule_configuration *mc = &mcs[MI_SCREEN(mi)];
   int wire = MI_IS_WIREFRAME(mi);
   molecule *m = &mc->molecules[mc->which];
-# ifdef HAVE_GLBITMAP
-  XFontStruct *xfont = (mc->scale_down ? mc->xfont2 : mc->xfont1);
-  GLuint font_dlist  = (mc->scale_down ? mc->font2_dlist : mc->font1_dlist);
-# else
-  texture_font_data *font_data = mc->font1_data;  /* don't scale down */
-# endif
   int i;
 
   if (!do_labels)
     return;
 
-  if (!wire)
-    glDisable (GL_LIGHTING);   /* don't light fonts */
-
   for (i = 0; i < m->natoms; i++)
     {
       molecule_atom *a = &m->atoms[i];
@@ -1424,7 +1401,7 @@ draw_labels (ModeInfo *mi)
 
       /* First, we translate the origin to the center of the atom.
 
-         Then we retrieve the prevailing modelview matrix (which
+         Then we retrieve the prevailing modelview matrix, which
          includes any rotation, wandering, and user-trackball-rolling
          of the scene.
 
@@ -1459,49 +1436,27 @@ draw_labels (ModeInfo *mi)
 
       glRotatef (current_device_rotation(), 0, 0, 1);  /* right side up */
 
-# ifdef HAVE_GLBITMAP
-      glRasterPos3f (0, 0, 0);                     /* draw text here */
+      {
+        XCharStruct e;
+        int w, h;
+        GLfloat s;
 
-      /* Before drawing the string, shift the origin to center
-         the text over the origin of the sphere. */
-      glBitmap (0, 0, 0, 0,
-                -string_width (xfont, a->label, 0) / 2,
-                -xfont->descent,
-                NULL);
-      {
-        int j;
-        for (j = 0; j < strlen(a->label); j++)
-          glCallList (font_dlist + (int)(a->label[j]));
-      }
-# else
-      {
-        int h;
-        int w = texture_string_width (font_data, a->label, &h);
-        GLfloat s = 1.0 / h;
-        GLfloat max = 18;   /* max point size to avoid pixellated text */
-        if (h > max) s *= max/h;
+        texture_string_metrics (mc->atom_font, a->label, &e, 0, 0);
+        w = e.width;
+        h = e.ascent + e.descent;
+
+        s = 1.0 / h;			/* Scale to unit */
+        s *= mc->overall_scale;		/* Scale to size of atom */
+        s *= 0.8;			/* Shrink a bit */
         glScalef (s, s, 1);
-        glTranslatef (-w/2, h*2/3, 0);
-        print_gl_string (mi->dpy,
-# ifdef HAVE_GLBITMAP
-                         xfont, font_dlist,
-# else
-                         font_data,
-# endif
-                         0, 0, 0, 0,
-                         a->label, False);
+        glTranslatef (-w/2, -h/2, 0);
+#ifndef HAVE_ANDROID   /* Doesn't work -- causes whole scene to be black */
+        print_texture_string (mc->atom_font, a->label);
+#endif
       }
-# endif
 
       glPopMatrix();
     }
-
-  /* More efficient to always call glEnable() with correct values
-     than to call glPushAttrib()/glPopAttrib(), since reading
-     attributes from GL does a round-trip and  stalls the pipeline.
-   */
-  if (!wire)
-    glEnable (GL_LIGHTING);
 }
 
 
@@ -1519,6 +1474,18 @@ pick_new_molecule (ModeInfo *mi, time_t last)
     {
       mc->which = random() % mc->nmolecules;
     }
+  else if (mc->next < 0)
+    {
+      mc->which--;
+      if (mc->which < 0) mc->which = mc->nmolecules-1;
+      mc->next = 0;
+    }
+  else if (mc->next > 0)
+    {
+      mc->which++;
+      if (mc->which >= mc->nmolecules) mc->which = 0;
+      mc->next = 0;
+    }
   else
     {
       int n = mc->which;
@@ -1600,7 +1567,7 @@ draw_molecule (ModeInfo *mi)
   if (!mc->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(mc->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *mc->glx_context);
 
   if (mc->draw_time == 0)
     {
@@ -1621,7 +1588,7 @@ draw_molecule (ModeInfo *mi)
             {
               /* randomize molecules every -timeout seconds */
               mc->mode = 1;    /* go out */
-              mc->mode_tick = 10 * speed;
+              mc->mode_tick = 80 / speed;
               mc->draw_time = now;
             }
         }
@@ -1630,10 +1597,9 @@ draw_molecule (ModeInfo *mi)
     {
       if (--mc->mode_tick <= 0)
         {
-          mc->mode_tick = 10 * speed;
+          mc->mode_tick = 80 / speed;
           mc->mode = 2;  /* go in */
           pick_new_molecule (mi, mc->draw_time);
-          mc->draw_time = now;
         }
     }
   else if (mc->mode == 2)   /* in */
@@ -1654,10 +1620,7 @@ draw_molecule (ModeInfo *mi)
                  (y - 0.5) * 9,
                  (z - 0.5) * 9);
 
-    /* Do it twice because we don't track the device's orientation. */
-    glRotatef( current_device_rotation(), 0, 0, 1);
     gltrackball_rotate (mc->trackball);
-    glRotatef(-current_device_rotation(), 0, 0, 1);
 
     get_rotation (mc->rot, &x, &y, &z, !mc->button_down_p);
     glRotatef (x * 360, 1.0, 0.0, 0.0);
@@ -1670,8 +1633,8 @@ draw_molecule (ModeInfo *mi)
   if (mc->mode != 0)
     {
       GLfloat s = (mc->mode == 1
-                   ? mc->mode_tick / (10 * speed)
-                   : ((10 * speed) - mc->mode_tick + 1) / (10 * speed));
+                   ? mc->mode_tick / (80 / speed)
+                   : ((80 / speed) - mc->mode_tick + 1) / (80 / speed));
       glScalef (s, s, s);
     }
 
@@ -1689,15 +1652,11 @@ draw_molecule (ModeInfo *mi)
       if (do_titles && m->label && *m->label)
         {
           set_atom_color (mi, 0, True, 1);
-          print_gl_string (mi->dpy,
-# ifdef HAVE_GLBITMAP
-                           mc->xfont3, mc->font3_dlist,
-# else
-                           mc->font3_data,
-# endif
-                           mi->xgwa.width, mi->xgwa.height,
-                           10, mi->xgwa.height - 10,
-                           m->label, False);
+#ifndef HAVE_ANDROID   /* Doesn't work -- causes whole scene to be black */
+          print_texture_label (mi->dpy, mc->title_font,
+                               mi->xgwa.width, mi->xgwa.height,
+                               1, m->label);
+#endif
         }
     }
   glPopMatrix();
@@ -1730,6 +1689,35 @@ draw_molecule (ModeInfo *mi)
   glXSwapBuffers(dpy, window);
 }
 
+
+ENTRYPOINT void
+free_molecule (ModeInfo *mi)
+{
+  molecule_configuration *mc = &mcs[MI_SCREEN(mi)];
+  int i, j;
+
+  if (!mc->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *mc->glx_context);
+
+  for (i = 0; i < mc->nmolecules; i++) {
+    molecule *m = &mc->molecules[i];
+    for (j = 0; j < m->natoms; j++)
+      free (m->atoms[j].label);
+    free (m->atoms);
+    free (m->bonds);
+    free (m->label);
+  }
+  if (mc->molecules) free (mc->molecules);
+  if (mc->trackball) gltrackball_free (mc->trackball);
+  if (mc->rot) free_rotator (mc->rot);
+  if (mc->atom_font) free_texture_font (mc->atom_font);
+  if (mc->title_font) free_texture_font (mc->title_font);
+
+  if (glIsList(mc->molecule_dlist)) glDeleteLists(mc->molecule_dlist, 1);
+  if (glIsList(mc->shell_dlist)) glDeleteLists(mc->shell_dlist, 1);
+  if (glIsList(mc->molecule_dlist)) glDeleteLists(mc->molecule_dlist, 1);
+}
+
 XSCREENSAVER_MODULE ("Molecule", molecule)
 
 #endif /* USE_GL */
