@@ -1,4 +1,4 @@
-/* sonar, Copyright (c) 1998-2012 Jamie Zawinski and Stephen Martin
+/* sonar, Copyright (c) 1998-2020 Jamie Zawinski and Stephen Martin
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -18,11 +18,15 @@
 
 #undef usleep /* conflicts with unistd.h on OSX */
 
-#ifdef USE_IPHONE
+#ifdef HAVE_IPHONE
   /* Note: to get this to compile for iPhone, you need to fix Xcode!
      The icmp headers exist for the simulator build environment, but
      not for the real-device build environment.  This appears to 
-     just be an Apple bug, not intentional.
+     just be an Apple bug, not intentional.  But it has existed for
+     many years.
+
+     The "ICMP Sanity Check" build phase in the Xcode project checks
+     this and tells you what to do.
 
      xc=/Applications/Xcode.app/Contents
      for path in    /Developer/Platforms/iPhone*?/Developer/SDKs/?* \
@@ -68,6 +72,9 @@
 # ifdef HAVE_GETIFADDRS
 #  include <ifaddrs.h>
 # endif
+# ifdef HAVE_LIBCAP
+#  include <sys/capability.h>
+# endif
 #endif /* HAVE_ICMP || HAVE_ICMPHDR */
 
 #if defined(HAVE_ICMP)
@@ -165,7 +172,7 @@ typedef struct {
 
 
 /* Packs an IP address quad into bigendian network order. */
-static unsigned long
+static in_addr_t
 pack_addr (unsigned int a, unsigned int b, unsigned int c, unsigned int d)
 {
   unsigned long i = (((a & 255) << 24) |
@@ -243,6 +250,7 @@ resolve_bogie_hostname (ping_data *pd, sonar_bogie *sb, Bool resolve_p)
       if (!strcmp (sb->name, "ssh-rsa") ||
           !strcmp (sb->name, "ssh-dsa") ||
           !strcmp (sb->name, "ssh-dss") ||
+          !strncmp (sb->name, "ecdsa-", 6) ||
           strlen (sb->name) >= 80)
         return 0;
 
@@ -448,12 +456,13 @@ read_hosts_file (sonar_sensor_data *ssd, const char *filename)
   fp = fopen(filename, "r");
   if (!fp)
     {
-      char buf[1024];
-      sprintf(buf, "%s:  %s", progname, filename);
 #ifdef HAVE_JWXYZ
       if (pd->debug_p)  /* on OSX don't syslog this */
 #endif
-        perror (buf);
+        {
+          char *str_error = strerror(errno);
+          fprintf(stderr, "%s:  %s: %s", progname, filename, str_error);
+        }
       return 0;
     }
 
@@ -640,10 +649,10 @@ delete_duplicate_hosts (sonar_sensor_data *ssd, sonar_bogie *list)
 }
 
 
-static unsigned int
-width_mask (int width)
+static unsigned long
+width_mask (unsigned long width)
 {
-  unsigned int m = 0;
+  unsigned long m = 0;
   int i;
   for (i = 0; i < width; i++)
     m |= (1L << (31-i));
@@ -652,8 +661,8 @@ width_mask (int width)
 
 
 #ifdef HAVE_GETIFADDRS
-static int
-mask_width (unsigned int mask)
+static unsigned int
+mask_width (unsigned long mask)
 {
   int i;
   for (i = 0; i < 32; i++)
@@ -726,7 +735,9 @@ subnet_hosts (sonar_sensor_data *ssd, char **error_ret, char **desc_ret,
         {
           struct in_addr in2;
           unsigned long mask;
-          if (ifa->ifa_addr->sa_family != AF_INET)
+          if (! ifa->ifa_addr)
+            continue;
+          else if (ifa->ifa_addr->sa_family != AF_INET)
             {
               if (pd->debug_p)
                 fprintf (stderr, "%s:     if: %4s: %s\n", progname,
@@ -755,7 +766,9 @@ subnet_hosts (sonar_sensor_data *ssd, char **error_ret, char **desc_ret,
                      mask_width (mask),
                      mask);
           if (in2.s_addr == 0x0100007f ||   /* 127.0.0.1 in network order */
+              ((in2.s_addr & 0x000000ff) == 0x7f) ||  /* 127.0.0.0/24 */
               mask == 0)
+            /* Assume all 127 addresses are loopback, not just 127.0.0.1. */
             continue;
 
           /* At least on the AT&T 3G network, pinging either of the two
@@ -777,6 +790,10 @@ subnet_hosts (sonar_sensor_data *ssd, char **error_ret, char **desc_ret,
 
           in = in2;
           subnet_width = mask_width (mask);
+
+          /* Take the first non-loopback network: prefer en0 over en1. */
+          if (in.s_addr && subnet_width)
+            break;
         }
 
       if (in.s_addr)
@@ -861,15 +878,15 @@ subnet_hosts (sonar_sensor_data *ssd, char **error_ret, char **desc_ret,
   h_base = ntohl (n_base);
 
   if (desc_ret && !*desc_ret) {
-    char buf[255];
+    char buf2[255];
     unsigned int a, b, c, d;
     unsigned long bb = n_base & htonl(h_mask);
     unpack_addr (bb, &a, &b, &c, &d);
     if (subnet_width > 24)
-      sprintf (buf, "%u.%u.%u.%u/%d", a, b, c, d, subnet_width);
+      sprintf (buf2, "%u.%u.%u.%u/%d", a, b, c, d, subnet_width);
     else
-      sprintf (buf, "%u.%u.%u/%d", a, b, c, subnet_width);
-    *desc_ret = strdup (buf);
+      sprintf (buf2, "%u.%u.%u/%d", a, b, c, subnet_width);
+    *desc_ret = strdup (buf2);
   }
 
   for (i = 255; i >= 0; i--) {
@@ -927,10 +944,11 @@ send_ping (ping_data *pd, const sonar_bogie *b)
   struct ICMP *icmph;
   const char *token = "org.jwz.xscreensaver.sonar";
   char *host_id;
+  struct timeval tval;
 
-  int pcktsiz = (sizeof(struct ICMP) + sizeof(struct timeval) + 
+  unsigned long pcktsiz = (sizeof(struct ICMP) + sizeof(struct timeval) +
                  sizeof(socklen_t) + pb->addrlen +
-                 strlen(token) + 1 + 
+                 strlen(token) + 1 +
                  strlen(pd->version) + 1);
 
   /* Create the ICMP packet */
@@ -944,12 +962,16 @@ send_ping (ping_data *pd, const sonar_bogie *b)
   ICMP_CHECKSUM(icmph) = 0;
   ICMP_ID(icmph) = pd->pid;
   ICMP_SEQ(icmph) = pd->seq++;
+  /* struct timeval needs alignment, so we first use aligned buffer for
+     gettimeofday() and later copy the result to packet buffer
+   */
 # ifdef GETTIMEOFDAY_TWO_ARGS
-  gettimeofday((struct timeval *) &packet[sizeof(struct ICMP)],
+  gettimeofday((struct timeval *) &tval,
                (struct timezone *) 0);
 # else
-  gettimeofday((struct timeval *) &packet[sizeof(struct ICMP)]);
+  gettimeofday((struct timeval *) &tval);
 # endif
+  memcpy(&packet[sizeof(struct ICMP)], &tval, sizeof tval);
 
   /* We store the sockaddr of the host we're pinging in the packet, and parse
      that out of the return packet later (see get_ping() for why).
@@ -977,6 +999,8 @@ send_ping (ping_data *pd, const sonar_bogie *b)
       perror(buf);
 #endif
     }
+
+  free (packet);
 }
 
 /* signal handler */
@@ -1050,11 +1074,11 @@ get_ping (sonar_sensor_data *ssd)
 {
   ping_data *pd = (ping_data *) ssd->closure;
   struct sockaddr from;
-  unsigned int fromlen;  /* Posix says socklen_t, but that's not portable */
+  socklen_t fromlen;
   int result;
   u_char packet[1024];
   struct timeval now;
-  struct timeval *then;
+  struct timeval then;
   struct ip *ip;
   int iphdrlen;
   struct ICMP *icmph;
@@ -1105,7 +1129,7 @@ get_ping (sonar_sensor_data *ssd)
          From Valentijn Sessink <valentyn@openoffice.nl> */
       if (select(pd->icmpsock + 1, &rfds, 0, 0, &tv) >0)
         {
-          result = recvfrom (pd->icmpsock, packet, sizeof(packet),
+          result = (int)recvfrom (pd->icmpsock, packet, sizeof(packet),
                              0, &from, &fromlen);
 
           /* Check the packet */
@@ -1118,7 +1142,10 @@ get_ping (sonar_sensor_data *ssd)
           ip = (struct ip *) packet;
           iphdrlen = IP_HDRLEN(ip) << 2;
           icmph = (struct ICMP *) &packet[iphdrlen];
-          then  = (struct timeval *) &packet[iphdrlen + sizeof(struct ICMP)];
+           /* struct timeval data in packet is not aligned, move the data to
+              the aligned buffer
+             */
+          memcpy(&then, &packet[iphdrlen + sizeof(struct ICMP)], sizeof then);
 
 
           /* Ignore anything but ICMP Replies */
@@ -1207,7 +1234,7 @@ get_ping (sonar_sensor_data *ssd)
           bl = new;
 
           {
-            double msec = delta(then, &now) / 1000.0;
+            double msec = delta(&then, &now) / 1000.0;
 
             if (pd->times_p)
               {
@@ -1226,7 +1253,7 @@ get_ping (sonar_sensor_data *ssd)
                 if (strlen(s) > 28)
                   {
                     s2 = s + strlen(s) - 28;
-                    strncpy (s2, "...", 3);
+                    memcpy (s2, "...", 3);
                   }
                 fprintf (stdout, 
                          "%3d bytes from %28s: icmp_seq=%-4d time=%s\n",
@@ -1373,13 +1400,15 @@ ping_scan (sonar_sensor_data *ssd)
                     {
                       sonar_bogie *new_bogie = bogie_for_host (ssd, fallback,
                                                                NULL);
-                      new_bogie->next = *sbp;
+                      if (new_bogie) {
+                        new_bogie->next = *sbp;
 
-                      if (! ((ping_bogie *)new_bogie->closure)->lookup_addr &&
-                        ! find_duplicate_host(pd, &pd->targets, new_bogie))
-                        *sbp = new_bogie;
-                      else
-                        sonar_free_bogie (ssd, new_bogie);
+                        if (! ((ping_bogie *)new_bogie->closure)->lookup_addr &&
+                            ! find_duplicate_host(pd, &pd->targets, new_bogie))
+                          *sbp = new_bogie;
+                        else
+                          sonar_free_bogie (ssd, new_bogie);
+                      }
 
                       free (fallback);
                     }
@@ -1470,7 +1499,7 @@ parse_mode (sonar_sensor_data *ssd, char **error_ret, char **desc_ret,
 
       if (!ping_works_p)
         {
-          *error_ret = strdup ("Sonar must be setuid to ping!\n"
+          *error_ret = strdup ("Sonar must be setuid or libcap to ping!\n"
                                "Running simulation instead.");
           return 0;
         }
@@ -1556,6 +1585,46 @@ parse_mode (sonar_sensor_data *ssd, char **error_ret, char **desc_ret,
 }
 
 
+static Bool
+set_net_raw_capalibity(int enable_p)
+{
+  Bool ret_status = False;
+# ifdef HAVE_LIBCAP
+  cap_t cap_status;
+  cap_value_t cap_value[] = { CAP_NET_RAW, };
+  cap_flag_value_t cap_flag_value;
+  cap_flag_value_t new_value = enable_p ? CAP_SET : CAP_CLEAR;
+
+  cap_status = cap_get_proc();
+  do {
+    cap_flag_value = CAP_CLEAR;
+    if (cap_get_flag (cap_status, CAP_NET_RAW, CAP_EFFECTIVE, &cap_flag_value))
+      break;
+    if (cap_flag_value == new_value)
+      {
+        ret_status = True;
+        break;
+      }
+
+    cap_set_flag (cap_status, CAP_EFFECTIVE, 1, cap_value, new_value);
+    if (!cap_set_proc(cap_status)) 
+      ret_status = True;
+  } while (0);
+
+  if (cap_status) cap_free (cap_status);
+# endif /* HAVE_LIBCAP */
+
+  return ret_status;
+}
+
+static Bool
+set_ping_capability (void)
+{
+  if (geteuid() == 0) return True;
+  return set_net_raw_capalibity (True);
+}
+
+
 sonar_sensor_data *
 sonar_init_ping (Display *dpy, char **error_ret, char **desc_ret,
                  const char *subnet, int timeout,
@@ -1602,6 +1671,10 @@ sonar_init_ping (Display *dpy, char **error_ret, char **desc_ret,
 
      On MacOS X, we can avoid the whole problem by using a
      non-privileged datagram instead of a raw socket.
+
+     On recent Linux systems (2012-ish?) we can avoid setuid by instead
+     using cap_set_flag(... CAP_NET_RAW). To make that call the executable
+     needs to have "sudo setcap cap_net_raw=p sonar" done to it first.
    */
   if (global_icmpsock)
     {
@@ -1615,7 +1688,7 @@ sonar_init_ping (Display *dpy, char **error_ret, char **desc_ret,
     {
       socket_initted_p = True;
     }
-  else if (geteuid() == 0 &&
+  else if (set_ping_capability() &&
            (pd->icmpsock = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) >= 0)
     {
       socket_initted_p = True;
@@ -1634,7 +1707,7 @@ sonar_init_ping (Display *dpy, char **error_ret, char **desc_ret,
     fprintf (stderr, "%s: unable to open icmp socket\n", progname);
 
   /* Disavow privs */
-  setuid(getuid());
+  if (setuid(getuid()) == -1) abort();
 
   pd->pid = getpid() & 0xFFFF;
   pd->seq = 0;
@@ -1667,7 +1740,7 @@ sonar_init_ping (Display *dpy, char **error_ret, char **desc_ret,
       if (! *error_ret)
         *error_ret = strdup ("No hosts to ping!\n"
                              "Simulating instead.");
-      if (pd) ping_free_data (ssd, pd);
+      ping_free_data (ssd, pd);
       if (ssd) free (ssd);
       return 0;
     }
