@@ -274,14 +274,14 @@ static inline void point2rgb(int depth, unsigned long c, int *r, int *g, int *b)
 #endif
 	break;
     case 16:
-	*b = (c & 0x1f) << 3; 
-	*g = ((c >> 5) & 0x3f) << 2;
-	*r = ((c >> 11) & 0x1f) << 3; 
+	*b = (int) (c & 0x1f) << 3;
+	*g = (int) ((c >> 5) & 0x3f) << 2;
+	*r = (int) ((c >> 11) & 0x1f) << 3;
 	break;
     case 15:
-	*b = (c & 0x1f) << 3;
-	*g = ((c >> 5) & 0x1f) << 3;
-	*r = ((c >> 10) & 0x1f) << 3;
+	*b = (int) (c & 0x1f) << 3;
+	*g = (int) ((c >> 5) & 0x1f) << 3;
+	*r = (int) ((c >> 10) & 0x1f) << 3;
 	break;
     }
 }
@@ -974,6 +974,13 @@ static void
 interaggregate_free (Display *dpy, Window window, void *closure)
 {
   struct state *st = (struct state *) closure;
+  if (st->f) {
+    free_circles (st->f);
+    if (st->f->off_img) free (st->f->off_img);
+    if (st->f->parsedcolors) free (st->f->parsedcolors);
+    free (st->f);
+  }
+  if (st->fgc) XFreeGC (st->dpy, st->fgc);
   free (st);
 }
 
