@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1992-2010 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1992-2020 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -112,6 +112,10 @@
 #include "vroot.h"
 #include "fps.h"
 
+#ifdef HAVE_RECORD_ANIM
+# include "recanim.h"
+#endif
+
 #ifndef _XSCREENSAVER_VROOT_H_
 # error Error!  You have an old version of vroot.h!  Check -I args.
 #endif /* _XSCREENSAVER_VROOT_H_ */
@@ -133,6 +137,9 @@ const char *progname;   /* used by hacks in error messages */
 const char *progclass;  /* used by ../utils/resources.c */
 Bool mono_p;		/* used by hacks */
 
+#ifdef EXIT_AFTER
+static time_t exit_after;	/* Exit gracefully after N seconds */
+#endif
 
 static XrmOptionDescRec default_options [] = {
   { "-root",	".root",		XrmoptionNoArg, "True" },
@@ -147,16 +154,24 @@ static XrmOptionDescRec default_options [] = {
 
 # ifdef DEBUG_PAIR
   { "-pair",	".pair",		XrmoptionNoArg, "True" },
-# endif /* DEBUG_PAIR */
+# endif
+# ifdef HAVE_RECORD_ANIM
+  { "-record-animation", ".recordAnim", XrmoptionSepArg, 0 },
+# endif
+# ifdef EXIT_AFTER
+  { "-exit-after",	".exitAfter",	XrmoptionSepArg, 0 },
+# endif
+
   { 0, 0, 0, 0 }
 };
 
 static char *default_defaults[] = {
   ".root:		false",
-  "*geometry:		600x480", /* this should be .geometry, but nooooo... */
+  "*geometry:		1280x720", /* this should be .geometry, but noooo... */
   "*mono:		false",
   "*installColormap:	false",
   "*doFPS:		false",
+  "*multiSample:	false",
   "*visualID:		default",
   "*windowID:		",
   "*desktopGrabber:	xscreensaver-getimage %s",
@@ -250,7 +265,7 @@ MapNotify_event_p (Display *dpy, XEvent *event, XPointer window)
 }
 
 
-static Atom XA_WM_PROTOCOLS, XA_WM_DELETE_WINDOW;
+static Atom XA_WM_PROTOCOLS, XA_WM_DELETE_WINDOW, XA_NET_WM_PID;
 
 /* Dead-trivial event handling: exits if "q" or "ESC" are typed.
    Exit if the WM_PROTOCOLS WM_DELETE_WINDOW ClientMessage is received.
@@ -463,6 +478,12 @@ screenhack_table_handle_events (Display *dpy,
       if (XtAppPending (app) & (XtIMTimer|XtIMAlternateInput))
         XtAppProcessEvent (app, XtIMTimer|XtIMAlternateInput);
     }
+
+# ifdef EXIT_AFTER
+  if (exit_after != 0 && time ((time_t *) 0) >= exit_after)
+    return False;
+# endif
+
   return True;
 }
 
@@ -476,15 +497,23 @@ usleep_and_process_events (Display *dpy,
                          , Window window2, fps_state *fpst2, void *closure2,
                            unsigned long delay2
 #endif
+# ifdef HAVE_RECORD_ANIM
+                         , record_anim_state *anim_state
+# endif
                            )
 {
   do {
-    unsigned long quantum = 100000;  /* 1/10th second */
+    unsigned long quantum = 33333;  /* 30 fps */
     if (quantum > delay) 
       quantum = delay;
     delay -= quantum;
 
     XSync (dpy, False);
+
+#ifdef HAVE_RECORD_ANIM
+    if (anim_state) screenhack_record_anim (anim_state);
+#endif
+
     if (quantum > 0)
       {
         usleep (quantum);
@@ -509,7 +538,7 @@ usleep_and_process_events (Display *dpy,
 static void
 screenhack_do_fps (Display *dpy, Window w, fps_state *fpst, void *closure)
 {
-  fps_compute (fpst, 0);
+  fps_compute (fpst, 0, -1);
   fps_draw (fpst);
 }
 
@@ -519,6 +548,9 @@ run_screenhack_table (Display *dpy,
                       Window window,
 # ifdef DEBUG_PAIR
                       Window window2,
+# endif
+# ifdef HAVE_RECORD_ANIM
+                      record_anim_state *anim_state,
 # endif
                       const struct xscreensaver_function_table *ft)
 {
@@ -534,10 +566,12 @@ run_screenhack_table (Display *dpy,
 
   void *closure = init_cb (dpy, window, ft->setup_arg);
   fps_state *fpst = fps_init (dpy, window);
+  unsigned long delay = 0;
 
 #ifdef DEBUG_PAIR
   void *closure2 = 0;
   fps_state *fpst2 = 0;
+  unsigned long delay2 = 0;
   if (window2) closure2 = init_cb (dpy, window2, ft->setup_arg);
   if (window2) fpst2 = fps_init (dpy, window2);
 #endif
@@ -549,32 +583,40 @@ run_screenhack_table (Display *dpy,
 
   while (1)
     {
-      unsigned long delay = ft->draw_cb (dpy, window, closure);
-#ifdef DEBUG_PAIR
-      unsigned long delay2 = 0;
-      if (window2) delay2 = ft->draw_cb (dpy, window2, closure2);
-#endif
-
-      if (fpst) fps_cb (dpy, window, fpst, closure);
-#ifdef DEBUG_PAIR
-      if (fpst2) fps_cb (dpy, window, fpst2, closure);
-#endif
-
       if (! usleep_and_process_events (dpy, ft,
                                        window, fpst, closure, delay
 #ifdef DEBUG_PAIR
                                        , window2, fpst2, closure2, delay2
+#endif
+#ifdef HAVE_RECORD_ANIM
+                                       , anim_state
 #endif
                                        ))
         break;
+
+      delay = ft->draw_cb (dpy, window, closure);
+#ifdef DEBUG_PAIR
+      delay2 = 0;
+      if (window2) delay2 = ft->draw_cb (dpy, window2, closure2);
+#endif
+
+      if (fpst) fps_cb (dpy, window, fpst, closure);
+#ifdef DEBUG_PAIR
+      if (fpst2) fps_cb (dpy, window2, fpst2, closure2);
+#endif
     }
 
+#ifdef HAVE_RECORD_ANIM
+  /* Exiting before target frames hit: write the video anyway. */
+  if (anim_state) screenhack_record_anim_free (anim_state);
+#endif
+
   ft->free_cb (dpy, window, closure);
-  if (fpst) fps_free (fpst);
+  if (fpst) ft->fps_free (fpst);
 
 #ifdef DEBUG_PAIR
   if (window2) ft->free_cb (dpy, window2, closure2);
-  if (window2) fps_free (fpst2);
+  if (fpst2) ft->fps_free (fpst2);
 #endif
 }
 
@@ -647,6 +689,7 @@ init_window (Display *dpy, Widget toplevel, const char *title)
 {
   Window window;
   XWindowAttributes xgwa;
+  long pid = getpid();
   XtPopup (toplevel, XtGrabNone);
   XtVaSetValues (toplevel, XtNtitle, title, NULL);
 
@@ -660,6 +703,8 @@ init_window (Display *dpy, Widget toplevel, const char *title)
   XChangeProperty (dpy, window, XA_WM_PROTOCOLS, XA_ATOM, 32,
                    PropModeReplace,
                    (unsigned char *) &XA_WM_DELETE_WINDOW, 1);
+  XChangeProperty (dpy, window, XA_NET_WM_PID, XA_CARDINAL, 32,
+                   PropModeReplace, (unsigned char *)&pid, 1);
 }
 
 
@@ -675,6 +720,9 @@ main (int argc, char **argv)
 # ifdef DEBUG_PAIR
   Window window2 = 0;
   Widget toplevel2 = 0;
+# endif
+# ifdef HAVE_RECORD_ANIM
+  record_anim_state *anim_state = 0;
 # endif
   XtAppContext app;
   Bool root_p;
@@ -722,17 +770,22 @@ main (int argc, char **argv)
 
   XA_WM_PROTOCOLS = XInternAtom (dpy, "WM_PROTOCOLS", False);
   XA_WM_DELETE_WINDOW = XInternAtom (dpy, "WM_DELETE_WINDOW", False);
+  XA_NET_WM_PID = XInternAtom (dpy, "_NET_WM_PID", False);
 
   {
     char *v = (char *) strdup(strchr(screensaver_id, ' '));
     char *s1, *s2, *s3, *s4;
+    const char *ot = get_string_resource (dpy, "title", "Title");
     s1 = (char *) strchr(v,  ' '); s1++;
     s2 = (char *) strchr(s1, ' ');
     s3 = (char *) strchr(v,  '('); s3++;
     s4 = (char *) strchr(s3, ')');
     *s2 = 0;
     *s4 = 0;
-    sprintf (version, "%s: from the XScreenSaver %s distribution (%s.)",
+    if (ot && !*ot) ot = 0;
+    sprintf (version, "%.50s%s%s: from the XScreenSaver %s distribution (%s)",
+             (ot ? ot : ""),
+             (ot ? ": " : ""),
 	     progclass, s1, s3);
     free(v);
   }
@@ -747,7 +800,7 @@ main (int argc, char **argv)
                      !strcmp(argv[1], "--help"));
       fprintf (stderr, "%s\n", version);
       for (s = progclass; *s; s++) fprintf(stderr, " ");
-      fprintf (stderr, "  http://www.jwz.org/xscreensaver/\n\n");
+      fprintf (stderr, "  https://www.jwz.org/xscreensaver/\n\n");
 
       if (!help_p)
 	fprintf(stderr, "Unrecognised option: %s\n", argv[1]);
@@ -826,6 +879,15 @@ main (int argc, char **argv)
 
   root_p = get_boolean_resource (dpy, "root", "Boolean");
 
+# ifdef EXIT_AFTER
+  {
+    int secs = get_integer_resource (dpy, "exitAfter", "Integer");
+    exit_after = (secs > 0
+                  ? time((time_t *) 0) + secs
+                  : 0);
+  }      
+# endif
+
   {
     char *s = get_string_resource (dpy, "windowID", "WindowID");
     if (s && *s)
@@ -917,12 +979,28 @@ main (int argc, char **argv)
 # undef ya_rand_init
   ya_rand_init (0);
 
+
+#ifdef HAVE_RECORD_ANIM
+  {
+    int frames = get_integer_resource (dpy, "recordAnim", "Integer");
+    if (frames > 0)
+      anim_state = screenhack_record_anim_init (xgwa.screen, window, frames);
+  }
+#endif
+
   run_screenhack_table (dpy, window, 
 # ifdef DEBUG_PAIR
                         window2,
+# endif
+# ifdef HAVE_RECORD_ANIM
+                        anim_state,
 # endif
                         ft);
 
+#ifdef HAVE_RECORD_ANIM
+  if (anim_state) screenhack_record_anim_free (anim_state);
+#endif
+
   XtDestroyWidget (toplevel);
   XtDestroyApplicationContext (app);
 
