@@ -372,6 +372,9 @@ initialize (struct state *st)
   printf("fast\t%i\nmarks\t%i\nwait\t%i\n",st->fastch,st->maxk,st->timewait);
 #endif
  
+  free (xgc);
+  free (xorgc);
+  free (xandgc);
 }
 
 static void fill_kugel(struct state *st, int i, Pixmap buf, int setcol);
@@ -527,7 +530,10 @@ static void
 fill_kugel(struct state *st, int i, Pixmap buf, int setcol)
 {
   double ra;
-  int m,col,inc=1,inr=3,d;
+  int m,col,inr=3,d;
+#ifdef USE_POLYGON
+  int inc=1;
+#endif
   d=(int)((ABS(st->kugeln[i].r1)*2));
   if (d==0) d=1;
   
@@ -557,10 +563,12 @@ fill_kugel(struct state *st, int i, Pixmap buf, int setcol)
 #ifdef PRTDBX
 	  printf("Radius: %f\n",ra);
 #endif
+#ifdef USE_POLYGON
 	  if(-ra< 3.0) inc=14;
 	  else if(-ra< 6.0) inc=8;
 	  else if(-ra<20.0) inc=4;
 	  else if(-ra<40.0) inc=2;
+#endif
 	  if(setcol)
 	    {
 	      if (m==27)
@@ -931,6 +939,13 @@ t3d_event (Display *dpy, Window window, void *closure, XEvent *event)
 static void
 t3d_free (Display *dpy, Window window, void *closure)
 {
+  struct state *st = (struct state *) closure;
+  XFreeGC (dpy, st->gc);
+  XFreeGC (dpy, st->orgc);
+  XFreeGC (dpy, st->andgc);
+  XFreePixmap (dpy, st->buffer);
+  free (st->zeit);
+  free (st);
 }
 
 
