@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1992-2016 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1992-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -44,7 +44,7 @@
 
        load_image_async CB
            load_random_image_cocoa
-               osx_grab_desktop_image (osxgrabscreen.m, MacOS version)
+               osx_grab_desktop_image (grabclient-osx.m, MacOS version)
                    copy_framebuffer_to_ximage
                    XPutImage
                draw_colorbars
@@ -62,7 +62,7 @@
 
        load_image_async CB
            load_random_image_cocoa
-               osx_grab_desktop_image (osxgrabscreen.m, iOS version)
+               osx_grab_desktop_image (grabclient-osx.m, iOS version)
                    CGWindowListCreateImage
                    jwxyz_draw_NSImage_or_CGImage
                draw_colorbars
@@ -72,7 +72,7 @@
                        CB name, geom, closure
 
 
-   Andrid execution path:
+   Android execution path:
 
        load_image_async CB
            load_random_image_android
@@ -127,11 +127,11 @@ typedef struct {
   XtInputId pipe_id;
   FILE *pipe;
 
-# if !defined(USE_IPHONE) && !defined(HAVE_COCOA)  /* Real X11 */
+# if !defined(HAVE_IPHONE) && !defined(HAVE_COCOA)  /* Real X11 */
   pid_t pid;
 # endif
 
-# if !defined(USE_IPHONE) && defined(HAVE_COCOA)   /* Desktop OSX */
+# if !defined(HAVE_IPHONE) && defined(HAVE_COCOA)   /* Desktop OSX */
   char *directory;
 # endif
 
@@ -564,7 +564,7 @@ load_random_image_x11 (Screen *screen, Window window, Drawable drawable,
 
 #elif defined (HAVE_COCOA) /* OSX or iOS */
 
-# ifndef USE_IPHONE   /* HAVE_COCOA && !USE_IPHONE -- desktop OSX */
+# ifndef HAVE_IPHONE   /* HAVE_COCOA && !HAVE_IPHONE -- desktop OSX */
 
 # define BACKSLASH(c) \
   (! ((c >= 'a' && c <= 'z') || \
@@ -671,7 +671,7 @@ xscreensaver_getimage_file_cb (XtPointer closure, int *source, XtInputId *id)
 
     XGetGeometry (dpy, clo2->drawable, &r, &x, &y, &w, &h, &bbw, &d);
     draw_colorbars (clo2->screen, xgwa.visual, clo2->drawable, xgwa.colormap,
-                    0, 0, w, h);
+                    0, 0, w, h, 0, 0); /* #### logo missing */
     geom.x = geom.y = 0;
     geom.width = w;
     geom.height = h;
@@ -699,10 +699,10 @@ xscreensaver_getimage_file_cb (XtPointer closure, int *source, XtInputId *id)
 }
 
 
-# else   /* HAVE_COCOA && USE_IPHONE -- iOS */
+# else   /* HAVE_COCOA && HAVE_IPHONE -- iOS */
 
 /* Callback for ios_load_random_image(), called after we have loaded an
-   image from the iOS device's Photo Library.  See iosgrabimage.m.
+   image from the iOS device's Photo Library.  See grabclient-ios.m.
  */
 static void
 ios_load_random_image_cb (void *uiimage, const char *filename, 
@@ -740,7 +740,7 @@ ios_load_random_image_cb (void *uiimage, const char *filename,
   else  /* Probably means no images in the gallery. */
     {
       draw_colorbars (clo2->screen, xgwa.visual, clo2->drawable, xgwa.colormap,
-                      0, 0, w, h);
+                      0, 0, w, h, 0, 0); /* #### logo missing */
       geom.x = geom.y = 0;
       geom.width = w;
       geom.height = h;
@@ -753,7 +753,7 @@ ios_load_random_image_cb (void *uiimage, const char *filename,
   free (clo2);
 }
 
-# endif /* HAVE_COCOA && USE_IPHONE */
+# endif /* HAVE_COCOA && HAVE_IPHONE */
 
 
 static void
@@ -774,7 +774,7 @@ osx_load_image_file_async (Screen *screen, Window xwindow, Drawable drawable,
   clo2->callback = callback;
   clo2->closure = closure;
 
-# ifndef USE_IPHONE   /* Desktop OSX */
+# ifndef HAVE_IPHONE   /* Desktop OSX */
   clo2->directory = strdup (dir);
   clo2->pipe = open_image_name_pipe (dir);
   clo2->pipe_id = XtAppAddInput (XtDisplayToApplicationContext (
@@ -782,9 +782,14 @@ osx_load_image_file_async (Screen *screen, Window xwindow, Drawable drawable,
                             fileno (clo2->pipe),
                             (XtPointer) (XtInputReadMask | XtInputExceptMask),
                             xscreensaver_getimage_file_cb, (XtPointer) clo2);
-# else /* USE_IPHONE */
-  ios_load_random_image (ios_load_random_image_cb, clo2);
-# endif /* USE_IPHONE */
+# else /* HAVE_IPHONE */
+  {
+    XWindowAttributes xgwa;
+    XGetWindowAttributes (DisplayOfScreen (screen), xwindow, &xgwa);
+    ios_load_random_image (ios_load_random_image_cb, clo2,
+                           xgwa.width, xgwa.height);
+  }
+# endif /* HAVE_IPHONE */
 }
 
 
@@ -823,13 +828,13 @@ load_random_image_cocoa (Screen *screen, Window window, Drawable drawable,
   geom.width  = xgwa.width;
   geom.height = xgwa.height;
 
-# ifndef USE_IPHONE
+# ifndef HAVE_IPHONE
   if (filep)
     dir = get_string_resource (dpy, "imageDirectory", "ImageDirectory");
 
   if (!dir || !*dir)
     filep = False;
-# endif /* ! USE_IPHONE */
+# endif /* ! HAVE_IPHONE */
 
   if (deskp && filep) {
     deskp = !(random() & 5);    /* if both, desktop 1/5th of the time */
@@ -850,8 +855,8 @@ load_random_image_cocoa (Screen *screen, Window window, Drawable drawable,
   }
 
   if (! done)
-    draw_colorbars (screen, xgwa.visual, drawable, xgwa.colormap,
-                    0, 0, xgwa.width, xgwa.height);
+    draw_colorbars (screen, xgwa.visual, drawable, xgwa.colormap, 0, 0,
+                    xgwa.width, xgwa.height, 0, 0); /* #### logo missing */
 
   /* If we got here, we loaded synchronously, so we're done. */
   callback (screen, window, drawable, name, &geom, closure);
@@ -873,10 +878,6 @@ load_random_image_android (Screen *screen, Window window, Drawable drawable,
   Display *dpy = DisplayOfScreen (screen);
   XWindowAttributes xgwa;
   XRectangle geom;
-  char *name = 0;
-  char *data = 0;
-  int width  = 0;
-  int height = 0;
   
   if (!drawable) abort();
 
@@ -895,26 +896,16 @@ load_random_image_android (Screen *screen, Window window, Drawable drawable,
   geom.width  = xgwa.width;
   geom.height = xgwa.height;
 
-  data = jwxyz_load_random_image (dpy, &width, &height, &name);
-  if (! data)
-    draw_colorbars (screen, xgwa.visual, drawable, xgwa.colormap,
-                    0, 0, xgwa.width, xgwa.height);
-  else
-    {
-      XImage *img = XCreateImage (dpy, xgwa.visual, 32,
-                                  ZPixmap, 0, data, width, height, 0, 0);
-      XGCValues gcv;
-      GC gc;
-      gcv.foreground = BlackPixelOfScreen (screen);
-      gc = XCreateGC (dpy, drawable, GCForeground, &gcv);
-      XFillRectangle (dpy, drawable, gc, 0, 0, xgwa.width, xgwa.height);
-      XPutImage (dpy, drawable, gc, img, 0, 0, 
-                 (xgwa.width  - width) / 2,
-                 (xgwa.height - height) / 2,
-                 width, height);
-      XDestroyImage (img);
-      XFreeGC (dpy, gc);
-    }
+  XGCValues gcv;
+  gcv.foreground = BlackPixelOfScreen (screen);
+  GC gc = XCreateGC (dpy, drawable, GCForeground, &gcv);
+  XFillRectangle (dpy, drawable, gc, 0, 0, xgwa.width, xgwa.height);
+  char *name = jwxyz_draw_random_image (dpy, drawable, gc);
+  if (! name) {
+    draw_colorbars (screen, xgwa.visual, drawable, xgwa.colormap, 0, 0,
+                    xgwa.width, xgwa.height, 0, 0); /* #### logo missing */
+  }
+  XFreeGC (dpy, gc);
 
   callback (screen, window, drawable, name, &geom, closure);
   if (name) free (name);
