@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2012-2016 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 2012-2020 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -22,7 +22,7 @@
 
 #include "utils.h"
 
-#if !defined(USE_IPHONE) && !defined(HAVE_ANDROID)  /* whole file */
+#if !defined(HAVE_IPHONE) && !defined(HAVE_ANDROID)  /* whole file */
 
 #include "textclient.h"
 #include "resources.h"
@@ -104,6 +104,85 @@ subproc_cb (XtPointer closure, int *source, XtInputId *id)
       (c >= '0' && c <= '9') || \
       c == '.' || c == '_' || c == '-' || c == '+' || c == '/'))
 
+#ifdef HAVE_COCOA
+static char *
+escape_str (char *s, const char *src)
+{
+  while (*src) {
+    char c = *src++;
+    if (BACKSLASH(c)) *s++ = '\\';
+    *s++ = c;
+  }
+  return s;
+}
+#endif
+
+
+/* Let's see if we're able to fork and exec at all. Thanks, macOS.
+ */
+static Bool
+selftest (void)
+{
+  static Bool done = False;
+  pid_t pid;
+  char buf [255];
+  if (done) return True;
+
+  pid = fork ();
+  switch ((int) pid)
+    {
+    case -1:
+      sprintf (buf, "%s: textclient: selftest: couldn't fork", progname);
+      perror (buf);
+      return False;
+
+    case 0:					/* child */
+      {
+        char * const av[] = { "/bin/sh", "-c", "true", 0 };
+        execvp (av[0], av);
+        exit (1);  /* exits child fork */
+        break;
+      }
+
+    default:					/* parent */
+      {
+        int status = -1;
+        int i;
+        /* Busy-loops are bad mmmmkayyyy */
+        struct timeval tv;
+        tv.tv_sec  = 0;
+        tv.tv_usec = 100000L;		/* 0.1 sec */
+        for (i = 0; i < 50; i++) {	/* 5 sec max */
+          pid_t pid2 = waitpid (pid, &status, 0);
+          if (pid == pid2) break;
+          (void) select (0, 0, 0, 0, &tv);
+        }
+
+        if (status != 0)
+          {
+# ifdef DEBUG
+            fprintf (stderr, "%s: selftest: textclient: status %d\n",
+                     progname, status);
+# endif
+            return False;
+          }
+        else
+          {
+# ifdef DEBUG
+            fprintf (stderr, "%s: textclient: selftest ok\n", progname);
+# endif
+            done = True;
+          }
+        break;
+      }
+    }
+
+  return True;
+}
+
+
+static void start_timer (text_data *d);
+
 static void
 launch_text_generator (text_data *d)
 {
@@ -112,23 +191,31 @@ launch_text_generator (text_data *d)
   const char *oprogram = d->program;
   char *s;
 
+  size_t oprogram_size = strlen(oprogram);
+  size_t len;
+
 # ifdef HAVE_COCOA
   /* /bin/sh on OS X 10.10 wipes out the PATH. */
   const char *path = getenv("PATH");
-  char *cmd = s = malloc ((strlen(oprogram) + strlen(path)) * 2 + 100);
+  size_t cmd_capacity = (oprogram_size + strlen(path)) * 2 + 100;
+  char *cmd = s = malloc (cmd_capacity);
   strcpy (s, "export PATH=");
   s += strlen (s);
-  while (*path) {
-    char c = *path++;
-    if (BACKSLASH(c)) *s++ = '\\';
-    *s++ = c;
-  }
+  s = escape_str (s, path);
   strcpy (s, "; ");
   s += strlen (s);
 # else
   char *cmd = s = malloc ((strlen(oprogram)) * 2 + 100);
 # endif
 
+  if (!selftest())
+    {
+      if (!d->out_buffer || !*d->out_buffer)
+        d->out_buffer = "Can't exec; Gatekeeper problem?\r\n\r\n";
+      start_timer (d);
+      return;
+    }
+
   strcpy (s, "( ");
   strcat (s, oprogram);
   s += strlen (s);
@@ -139,20 +226,82 @@ launch_text_generator (text_data *d)
      "xscreensaver-text --cols %d", but that makes things blow up
      if someone ever uses a --program that includes a % anywhere.
    */
-  if (!strcmp (oprogram, "xscreensaver-text"))
+  len = 17; /* strlen("xscreensaver-text") */
+  if (oprogram_size >= len &&
+    !memcmp (oprogram, "xscreensaver-text", len) &&
+    (oprogram[len] == ' ' || !oprogram[len]))
     {
-      if (d->char_w)
+      /* strstr is sloppy here. Technically, we should be parsing the command
+         line to identify flags and their arguments. This will blow up if one
+         of those pesky end users could set .textLiteral to "--cols".
+       */
+      if (d->char_w && !strstr (oprogram, "--cols "))
         sprintf (s, " --cols %d", d->char_w);
-      if (d->max_lines)
+      if (d->max_lines && !strstr (oprogram, "--lines "))
         sprintf (s, " --lines %d", d->max_lines);
       s += strlen(s);
+
+# ifdef HAVE_COCOA
+      /* Also special-case "xscreensaver-text" to specify the text content on
+         the command line. defaults(1) on macOS doesn't know about the default
+         screenhack resources that don't make it into the
+         ~/Library/Preferences/ByHost/org.jwz.xscreensaver.*.plist.
+       */
+
+      char *text_mode_flag = " --date";
+      char *value_res = NULL;
+      char *text_mode = get_string_resource (d->dpy, "textMode", "String");
+
+      if (text_mode)
+        {
+          if (!strcmp (text_mode, "1") || !strcmp (text_mode, "literal"))
+            {
+              text_mode_flag = " --text";
+              value_res = "textLiteral";
+            }
+          else if (!strcmp (text_mode, "2") || !strcmp (text_mode, "file"))
+            {
+              text_mode_flag = " --file";
+              value_res = "textFile";
+            }
+          else if (!strcmp (text_mode, "3") || !strcmp (text_mode, "url"))
+            {
+              text_mode_flag = " --url";
+              value_res = "textURL";
+            }
+          else if (!strcmp (text_mode, "4") || !strcmp (text_mode, "program"))
+            {
+              text_mode_flag = " --program";
+              value_res = "textProgram";
+            }
+
+          free (text_mode);
+        }
+
+      strcpy (s, text_mode_flag);
+      s += strlen (s);
+
+      if (value_res)
+        {
+          size_t old_s = s - cmd;
+          char *value = get_string_resource (d->dpy, value_res, "");
+          if (!value)
+            value = strdup("");
+          cmd = realloc(cmd, cmd_capacity + strlen(value) * 2);
+          s = cmd + old_s;
+          *s = ' ';
+          ++s;
+          s = escape_str(s, value);
+          free(value);
+        }
+# endif /* HAVE_COCOA */
     }
 
   strcpy (s, " ) 2>&1");
 
 # ifdef DEBUG
-  fprintf (stderr, "%s: textclient: launch %s: %s\n", cmd, 
-           (d->pty_p ? "pty" : "pipe"), program);
+  fprintf (stderr, "%s: textclient: launch %s: %s\n", progname, 
+           (d->pty_p ? "pty" : "pipe"), cmd);
 # endif
 
 #ifdef HAVE_FORKPTY
@@ -167,6 +316,15 @@ launch_text_generator (text_data *d)
       ws.ws_ypixel = d->pix_h;
       
       d->pipe = 0;
+
+# ifdef HAVE_COCOA
+      if (getenv ("MallocScribble"))
+        /* This is here to stop me from wasting my time trying to answer
+           this question the next time I forget about it. */
+        fprintf (stderr, "%s: WARNING: forkpty hates 'Enable Guard Malloc'\n",
+                 progname);
+# endif
+
       if ((d->pid = forkpty(&fd, NULL, NULL, &ws)) < 0)
 	{
           /* Unable to fork */
@@ -184,6 +342,15 @@ launch_text_generator (text_data *d)
           av[i++] = "-c";
           av[i++] = cmd;
           av[i] = 0;
+# ifdef DEBUG
+          {
+            int j;
+            fprintf (stderr, "%s: textclient: execvp:", progname);
+            for (j = 0; j < i; j++)
+              fprintf (stderr, " %s", av[j]);
+            fprintf (stderr, "\n");
+          }
+# endif
           execvp (av[0], av);
           sprintf (buf, "%.100s: %.100s", progname, oprogram);
 	  perror (buf);
@@ -327,6 +494,9 @@ textclient_reshape (text_data *d,
       ws.ws_ypixel = pix_h;
       ioctl (fileno (d->pipe), TIOCSWINSZ, &ws);
       kill (d->pid, SIGWINCH);
+#  ifdef DEBUG
+      fprintf (stderr, "%s: textclient: SIGWINCH\n", progname);
+#  endif
     }
 # endif /* HAVE_FORKPTY && TIOCSWINSZ */
 
@@ -337,6 +507,9 @@ textclient_reshape (text_data *d,
    */
   if (!strcmp (d->program, "xscreensaver-text"))
     {
+# ifdef DEBUG
+      fprintf (stderr, "%s: textclient: reshape relaunch\n", progname);
+# endif
       close_pipe (d);
       d->input_available_p = False;
       start_timer (d);
@@ -387,6 +560,10 @@ textclient_open (Display *dpy)
       {
         d->pty_p = 1;
         d->program = strdup (getenv ("SHELL"));
+#  ifdef DEBUG
+        fprintf (stderr, "%s: textclient: standalone: %s\n",
+                 progname, d->program);
+#  endif
       }
   }
 # endif
@@ -567,4 +744,4 @@ textclient_putc (text_data *d, XKeyEvent *k)
   return False;
 }
 
-#endif /* !USE_IPHONE -- whole file */
+#endif /* !HAVE_IPHONE -- whole file */
